// This file is auto-generated. Do not edit!

#include "opencv2/core/ocl.hpp"
#include "opencv2/core/ocl_genbase.hpp"
#include "opencv2/core/opencl/ocl_defs.hpp"

#ifdef HAVE_OPENCL

namespace cv
{
namespace ocl
{
namespace stitching
{

extern struct cv::ocl::internal::ProgramEntry multibandblend_oclsrc;
extern struct cv::ocl::internal::ProgramEntry warpers_oclsrc;

}}}
#endif
