// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: tensor.proto
// Protobuf C++ Version: 6.32.1

#include "tensor.pb.h"

#include <algorithm>
#include <type_traits>
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/generated_message_tctable_impl.h"
#include "google/protobuf/extension_set.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/wire_format_lite.h"
#include "google/protobuf/descriptor.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/reflection_ops.h"
#include "google/protobuf/wire_format.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"
PROTOBUF_PRAGMA_INIT_SEG
namespace _pb = ::google::protobuf;
namespace _pbi = ::google::protobuf::internal;
namespace _fl = ::google::protobuf::internal::field_layout;
namespace opencv_tensorflow {

inline constexpr TensorProto::Impl_::Impl_(
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        float_val_{},
        double_val_{},
        int_val_{},
        _int_val_cached_byte_size_{0},
        string_val_{},
        scomplex_val_{},
        int64_val_{},
        _int64_val_cached_byte_size_{0},
        bool_val_{},
        dcomplex_val_{},
        half_val_{},
        _half_val_cached_byte_size_{0},
        tensor_content_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        tensor_shape_{nullptr},
        dtype_{static_cast< ::opencv_tensorflow::DataType >(0)},
        version_number_{0} {}

template <typename>
PROTOBUF_CONSTEXPR TensorProto::TensorProto(::_pbi::ConstantInitialized)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(TensorProto_class_data_.base()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(::_pbi::ConstantInitialized()) {
}
struct TensorProtoDefaultTypeInternal {
  PROTOBUF_CONSTEXPR TensorProtoDefaultTypeInternal() : _instance(::_pbi::ConstantInitialized{}) {}
  ~TensorProtoDefaultTypeInternal() {}
  union {
    TensorProto _instance;
  };
};

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 TensorProtoDefaultTypeInternal _TensorProto_default_instance_;
}  // namespace opencv_tensorflow
static constexpr const ::_pb::EnumDescriptor *PROTOBUF_NONNULL *PROTOBUF_NULLABLE
    file_level_enum_descriptors_tensor_2eproto = nullptr;
static constexpr const ::_pb::ServiceDescriptor *PROTOBUF_NONNULL *PROTOBUF_NULLABLE
    file_level_service_descriptors_tensor_2eproto = nullptr;
const ::uint32_t
    TableStruct_tensor_2eproto::offsets[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
        protodesc_cold) = {
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::opencv_tensorflow::TensorProto, _impl_._has_bits_),
        16, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::opencv_tensorflow::TensorProto, _impl_.dtype_),
        PROTOBUF_FIELD_OFFSET(::opencv_tensorflow::TensorProto, _impl_.tensor_shape_),
        PROTOBUF_FIELD_OFFSET(::opencv_tensorflow::TensorProto, _impl_.version_number_),
        PROTOBUF_FIELD_OFFSET(::opencv_tensorflow::TensorProto, _impl_.tensor_content_),
        PROTOBUF_FIELD_OFFSET(::opencv_tensorflow::TensorProto, _impl_.half_val_),
        PROTOBUF_FIELD_OFFSET(::opencv_tensorflow::TensorProto, _impl_.float_val_),
        PROTOBUF_FIELD_OFFSET(::opencv_tensorflow::TensorProto, _impl_.double_val_),
        PROTOBUF_FIELD_OFFSET(::opencv_tensorflow::TensorProto, _impl_.int_val_),
        PROTOBUF_FIELD_OFFSET(::opencv_tensorflow::TensorProto, _impl_.string_val_),
        PROTOBUF_FIELD_OFFSET(::opencv_tensorflow::TensorProto, _impl_.scomplex_val_),
        PROTOBUF_FIELD_OFFSET(::opencv_tensorflow::TensorProto, _impl_.int64_val_),
        PROTOBUF_FIELD_OFFSET(::opencv_tensorflow::TensorProto, _impl_.bool_val_),
        PROTOBUF_FIELD_OFFSET(::opencv_tensorflow::TensorProto, _impl_.dcomplex_val_),
        2,
        1,
        3,
        0,
        ~0u,
        ~0u,
        ~0u,
        ~0u,
        ~0u,
        ~0u,
        ~0u,
        ~0u,
        ~0u,
};

static const ::_pbi::MigrationSchema
    schemas[] ABSL_ATTRIBUTE_SECTION_VARIABLE(protodesc_cold) = {
        {0, sizeof(::opencv_tensorflow::TensorProto)},
};
static const ::_pb::Message* PROTOBUF_NONNULL const file_default_instances[] = {
    &::opencv_tensorflow::_TensorProto_default_instance_._instance,
};
const char descriptor_table_protodef_tensor_2eproto[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
    protodesc_cold) = {
    "\n\014tensor.proto\022\021opencv_tensorflow\032\022tenso"
    "r_shape.proto\032\013types.proto\"\363\002\n\013TensorPro"
    "to\022*\n\005dtype\030\001 \001(\0162\033.opencv_tensorflow.Da"
    "taType\0229\n\014tensor_shape\030\002 \001(\0132#.opencv_te"
    "nsorflow.TensorShapeProto\022\026\n\016version_num"
    "ber\030\003 \001(\005\022\026\n\016tensor_content\030\004 \001(\014\022\024\n\010hal"
    "f_val\030\r \003(\005B\002\020\001\022\025\n\tfloat_val\030\005 \003(\002B\002\020\001\022\026"
    "\n\ndouble_val\030\006 \003(\001B\002\020\001\022\023\n\007int_val\030\007 \003(\005B"
    "\002\020\001\022\022\n\nstring_val\030\010 \003(\014\022\030\n\014scomplex_val\030"
    "\t \003(\002B\002\020\001\022\025\n\tint64_val\030\n \003(\003B\002\020\001\022\024\n\010bool"
    "_val\030\013 \003(\010B\002\020\001\022\030\n\014dcomplex_val\030\014 \003(\001B\002\020\001"
    "B-\n\030org.tensorflow.frameworkB\014TensorProt"
    "osP\001\370\001\001b\006proto3"
};
static const ::_pbi::DescriptorTable* PROTOBUF_NONNULL const
    descriptor_table_tensor_2eproto_deps[2] = {
        &::descriptor_table_tensor_5fshape_2eproto,
        &::descriptor_table_types_2eproto,
};
static ::absl::once_flag descriptor_table_tensor_2eproto_once;
PROTOBUF_CONSTINIT const ::_pbi::DescriptorTable descriptor_table_tensor_2eproto = {
    false,
    false,
    495,
    descriptor_table_protodef_tensor_2eproto,
    "tensor.proto",
    &descriptor_table_tensor_2eproto_once,
    descriptor_table_tensor_2eproto_deps,
    2,
    1,
    schemas,
    file_default_instances,
    TableStruct_tensor_2eproto::offsets,
    file_level_enum_descriptors_tensor_2eproto,
    file_level_service_descriptors_tensor_2eproto,
};
namespace opencv_tensorflow {
// ===================================================================

class TensorProto::_Internal {
 public:
  using HasBits =
      decltype(::std::declval<TensorProto>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(TensorProto, _impl_._has_bits_);
};

void TensorProto::clear_tensor_shape() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.tensor_shape_ != nullptr) _impl_.tensor_shape_->Clear();
  _impl_._has_bits_[0] &= ~0x00000002U;
}
TensorProto::TensorProto(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, TensorProto_class_data_.base()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:opencv_tensorflow.TensorProto)
}
PROTOBUF_NDEBUG_INLINE TensorProto::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::opencv_tensorflow::TensorProto& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        float_val_{visibility, arena, from.float_val_},
        double_val_{visibility, arena, from.double_val_},
        int_val_{visibility, arena, from.int_val_},
        _int_val_cached_byte_size_{0},
        string_val_{visibility, arena, from.string_val_},
        scomplex_val_{visibility, arena, from.scomplex_val_},
        int64_val_{visibility, arena, from.int64_val_},
        _int64_val_cached_byte_size_{0},
        bool_val_{visibility, arena, from.bool_val_},
        dcomplex_val_{visibility, arena, from.dcomplex_val_},
        half_val_{visibility, arena, from.half_val_},
        _half_val_cached_byte_size_{0},
        tensor_content_(arena, from.tensor_content_) {}

TensorProto::TensorProto(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const TensorProto& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, TensorProto_class_data_.base()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  TensorProto* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.tensor_shape_ = ((cached_has_bits & 0x00000002U) != 0)
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.tensor_shape_)
                : nullptr;
  ::memcpy(reinterpret_cast<char *>(&_impl_) +
               offsetof(Impl_, dtype_),
           reinterpret_cast<const char *>(&from._impl_) +
               offsetof(Impl_, dtype_),
           offsetof(Impl_, version_number_) -
               offsetof(Impl_, dtype_) +
               sizeof(Impl_::version_number_));

  // @@protoc_insertion_point(copy_constructor:opencv_tensorflow.TensorProto)
}
PROTOBUF_NDEBUG_INLINE TensorProto::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        float_val_{visibility, arena},
        double_val_{visibility, arena},
        int_val_{visibility, arena},
        _int_val_cached_byte_size_{0},
        string_val_{visibility, arena},
        scomplex_val_{visibility, arena},
        int64_val_{visibility, arena},
        _int64_val_cached_byte_size_{0},
        bool_val_{visibility, arena},
        dcomplex_val_{visibility, arena},
        half_val_{visibility, arena},
        _half_val_cached_byte_size_{0},
        tensor_content_(arena) {}

inline void TensorProto::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char *>(&_impl_) +
               offsetof(Impl_, tensor_shape_),
           0,
           offsetof(Impl_, version_number_) -
               offsetof(Impl_, tensor_shape_) +
               sizeof(Impl_::version_number_));
}
TensorProto::~TensorProto() {
  // @@protoc_insertion_point(destructor:opencv_tensorflow.TensorProto)
  SharedDtor(*this);
}
inline void TensorProto::SharedDtor(MessageLite& self) {
  TensorProto& this_ = static_cast<TensorProto&>(self);
  if constexpr (::_pbi::DebugHardenVerifyHasBitConsistency()) {
    this_.VerifyHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.tensor_content_.Destroy();
  delete this_._impl_.tensor_shape_;
  this_._impl_.~Impl_();
}

inline void* PROTOBUF_NONNULL TensorProto::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) TensorProto(arena);
}
constexpr auto TensorProto::InternalNewImpl_() {
  constexpr auto arena_bits = ::google::protobuf::internal::EncodePlacementArenaOffsets({
      PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.half_val_) +
          decltype(TensorProto::_impl_.half_val_)::
              InternalGetArenaOffset(
                  ::google::protobuf::Message::internal_visibility()),
      PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.float_val_) +
          decltype(TensorProto::_impl_.float_val_)::
              InternalGetArenaOffset(
                  ::google::protobuf::Message::internal_visibility()),
      PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.double_val_) +
          decltype(TensorProto::_impl_.double_val_)::
              InternalGetArenaOffset(
                  ::google::protobuf::Message::internal_visibility()),
      PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.int_val_) +
          decltype(TensorProto::_impl_.int_val_)::
              InternalGetArenaOffset(
                  ::google::protobuf::Message::internal_visibility()),
      PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.string_val_) +
          decltype(TensorProto::_impl_.string_val_)::
              InternalGetArenaOffset(
                  ::google::protobuf::Message::internal_visibility()),
      PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.scomplex_val_) +
          decltype(TensorProto::_impl_.scomplex_val_)::
              InternalGetArenaOffset(
                  ::google::protobuf::Message::internal_visibility()),
      PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.int64_val_) +
          decltype(TensorProto::_impl_.int64_val_)::
              InternalGetArenaOffset(
                  ::google::protobuf::Message::internal_visibility()),
      PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.bool_val_) +
          decltype(TensorProto::_impl_.bool_val_)::
              InternalGetArenaOffset(
                  ::google::protobuf::Message::internal_visibility()),
      PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.dcomplex_val_) +
          decltype(TensorProto::_impl_.dcomplex_val_)::
              InternalGetArenaOffset(
                  ::google::protobuf::Message::internal_visibility()),
  });
  if (arena_bits.has_value()) {
    return ::google::protobuf::internal::MessageCreator::CopyInit(
        sizeof(TensorProto), alignof(TensorProto), *arena_bits);
  } else {
    return ::google::protobuf::internal::MessageCreator(&TensorProto::PlacementNew_,
                                 sizeof(TensorProto),
                                 alignof(TensorProto));
  }
}
constexpr auto TensorProto::InternalGenerateClassData_() {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &_TensorProto_default_instance_._instance,
          &_table_.header,
          nullptr,  // OnDemandRegisterArenaDtor
          nullptr,  // IsInitialized
          &TensorProto::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<TensorProto>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &TensorProto::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<TensorProto>(), &TensorProto::ByteSizeLong,
              &TensorProto::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(TensorProto, _impl_._cached_size_),
          false,
      },
      &TensorProto::kDescriptorMethods,
      &descriptor_table_tensor_2eproto,
      nullptr,  // tracker
  };
}

PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull TensorProto_class_data_ =
        TensorProto::InternalGenerateClassData_();

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
TensorProto::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&TensorProto_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(TensorProto_class_data_.tc_table);
  return TensorProto_class_data_.base();
}
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1
const ::_pbi::TcParseTable<4, 13, 1, 0, 2>
TensorProto::_table_ = {
  {
    PROTOBUF_FIELD_OFFSET(TensorProto, _impl_._has_bits_),
    0, // no _extensions_
    13, 120,  // max_field_number, fast_idx_mask
    offsetof(decltype(_table_), field_lookup_table),
    4294959104,  // skipmap
    offsetof(decltype(_table_), field_entries),
    13,  // num_field_entries
    1,  // num_aux_entries
    offsetof(decltype(_table_), aux_entries),
    TensorProto_class_data_.base(),
    nullptr,  // post_loop_handler
    ::_pbi::TcParser::GenericFallback,  // fallback
    #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
    ::_pbi::TcParser::GetTable<::opencv_tensorflow::TensorProto>(),  // to_prefetch
    #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
  }, {{
    {::_pbi::TcParser::MiniParse, {}},
    // .opencv_tensorflow.DataType dtype = 1;
    {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(TensorProto, _impl_.dtype_), 2>(),
     {8, 2, 0, PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.dtype_)}},
    // .opencv_tensorflow.TensorShapeProto tensor_shape = 2;
    {::_pbi::TcParser::FastMtS1,
     {18, 1, 0, PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.tensor_shape_)}},
    // int32 version_number = 3;
    {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(TensorProto, _impl_.version_number_), 3>(),
     {24, 3, 0, PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.version_number_)}},
    // bytes tensor_content = 4;
    {::_pbi::TcParser::FastBS1,
     {34, 0, 0, PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.tensor_content_)}},
    // repeated float float_val = 5 [packed = true];
    {::_pbi::TcParser::FastF32P1,
     {42, 63, 0, PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.float_val_)}},
    // repeated double double_val = 6 [packed = true];
    {::_pbi::TcParser::FastF64P1,
     {50, 63, 0, PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.double_val_)}},
    // repeated int32 int_val = 7 [packed = true];
    {::_pbi::TcParser::FastV32P1,
     {58, 63, 0, PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.int_val_)}},
    // repeated bytes string_val = 8;
    {::_pbi::TcParser::FastBR1,
     {66, 63, 0, PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.string_val_)}},
    // repeated float scomplex_val = 9 [packed = true];
    {::_pbi::TcParser::FastF32P1,
     {74, 63, 0, PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.scomplex_val_)}},
    // repeated int64 int64_val = 10 [packed = true];
    {::_pbi::TcParser::FastV64P1,
     {82, 63, 0, PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.int64_val_)}},
    // repeated bool bool_val = 11 [packed = true];
    {::_pbi::TcParser::FastV8P1,
     {90, 63, 0, PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.bool_val_)}},
    // repeated double dcomplex_val = 12 [packed = true];
    {::_pbi::TcParser::FastF64P1,
     {98, 63, 0, PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.dcomplex_val_)}},
    // repeated int32 half_val = 13 [packed = true];
    {::_pbi::TcParser::FastV32P1,
     {106, 63, 0, PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.half_val_)}},
    {::_pbi::TcParser::MiniParse, {}},
    {::_pbi::TcParser::MiniParse, {}},
  }}, {{
    65535, 65535
  }}, {{
    // .opencv_tensorflow.DataType dtype = 1;
    {PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.dtype_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
    // .opencv_tensorflow.TensorShapeProto tensor_shape = 2;
    {PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.tensor_shape_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
    // int32 version_number = 3;
    {PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.version_number_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
    // bytes tensor_content = 4;
    {PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.tensor_content_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kBytes | ::_fl::kRepAString)},
    // repeated float float_val = 5 [packed = true];
    {PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.float_val_), -1, 0, (0 | ::_fl::kFcRepeated | ::_fl::kPackedFloat)},
    // repeated double double_val = 6 [packed = true];
    {PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.double_val_), -1, 0, (0 | ::_fl::kFcRepeated | ::_fl::kPackedDouble)},
    // repeated int32 int_val = 7 [packed = true];
    {PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.int_val_), -1, 0, (0 | ::_fl::kFcRepeated | ::_fl::kPackedInt32)},
    // repeated bytes string_val = 8;
    {PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.string_val_), -1, 0, (0 | ::_fl::kFcRepeated | ::_fl::kBytes | ::_fl::kRepSString)},
    // repeated float scomplex_val = 9 [packed = true];
    {PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.scomplex_val_), -1, 0, (0 | ::_fl::kFcRepeated | ::_fl::kPackedFloat)},
    // repeated int64 int64_val = 10 [packed = true];
    {PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.int64_val_), -1, 0, (0 | ::_fl::kFcRepeated | ::_fl::kPackedInt64)},
    // repeated bool bool_val = 11 [packed = true];
    {PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.bool_val_), -1, 0, (0 | ::_fl::kFcRepeated | ::_fl::kPackedBool)},
    // repeated double dcomplex_val = 12 [packed = true];
    {PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.dcomplex_val_), -1, 0, (0 | ::_fl::kFcRepeated | ::_fl::kPackedDouble)},
    // repeated int32 half_val = 13 [packed = true];
    {PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.half_val_), -1, 0, (0 | ::_fl::kFcRepeated | ::_fl::kPackedInt32)},
  }},
  {{
      {::_pbi::TcParser::GetTable<::opencv_tensorflow::TensorShapeProto>()},
  }},
  {{
  }},
};
PROTOBUF_NOINLINE void TensorProto::Clear() {
// @@protoc_insertion_point(message_clear_start:opencv_tensorflow.TensorProto)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  _impl_.float_val_.Clear();
  _impl_.double_val_.Clear();
  _impl_.int_val_.Clear();
  _impl_.string_val_.Clear();
  _impl_.scomplex_val_.Clear();
  _impl_.int64_val_.Clear();
  _impl_.bool_val_.Clear();
  _impl_.dcomplex_val_.Clear();
  _impl_.half_val_.Clear();
  cached_has_bits = _impl_._has_bits_[0];
  if ((cached_has_bits & 0x00000003U) != 0) {
    if ((cached_has_bits & 0x00000001U) != 0) {
      _impl_.tensor_content_.ClearNonDefaultToEmpty();
    }
    if ((cached_has_bits & 0x00000002U) != 0) {
      ABSL_DCHECK(_impl_.tensor_shape_ != nullptr);
      _impl_.tensor_shape_->Clear();
    }
  }
  if ((cached_has_bits & 0x0000000cU) != 0) {
    ::memset(&_impl_.dtype_, 0, static_cast<::size_t>(
        reinterpret_cast<char*>(&_impl_.version_number_) -
        reinterpret_cast<char*>(&_impl_.dtype_)) + sizeof(_impl_.version_number_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL TensorProto::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const TensorProto& this_ = static_cast<const TensorProto&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL TensorProto::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const TensorProto& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenVerifyHasBitConsistency()) {
    this_.VerifyHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:opencv_tensorflow.TensorProto)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  // .opencv_tensorflow.DataType dtype = 1;
  if ((this_._impl_._has_bits_[0] & 0x00000004U) != 0) {
    if (this_._internal_dtype() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          1, this_._internal_dtype(), target);
    }
  }

  cached_has_bits = this_._impl_._has_bits_[0];
  // .opencv_tensorflow.TensorShapeProto tensor_shape = 2;
  if ((cached_has_bits & 0x00000002U) != 0) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.tensor_shape_, this_._impl_.tensor_shape_->GetCachedSize(), target,
        stream);
  }

  // int32 version_number = 3;
  if ((cached_has_bits & 0x00000008U) != 0) {
    if (this_._internal_version_number() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<3>(
              stream, this_._internal_version_number(), target);
    }
  }

  // bytes tensor_content = 4;
  if ((cached_has_bits & 0x00000001U) != 0) {
    if (!this_._internal_tensor_content().empty()) {
      const ::std::string& _s = this_._internal_tensor_content();
      target = stream->WriteBytesMaybeAliased(4, _s, target);
    }
  }

  // repeated float float_val = 5 [packed = true];
  if (this_._internal_float_val_size() > 0) {
    target = stream->WriteFixedPacked(5, this_._internal_float_val(), target);
  }

  // repeated double double_val = 6 [packed = true];
  if (this_._internal_double_val_size() > 0) {
    target = stream->WriteFixedPacked(6, this_._internal_double_val(), target);
  }

  // repeated int32 int_val = 7 [packed = true];
  {
    int byte_size = this_._impl_._int_val_cached_byte_size_.Get();
    if (byte_size > 0) {
      target = stream->WriteInt32Packed(
          7, this_._internal_int_val(), byte_size, target);
    }
  }

  // repeated bytes string_val = 8;
  for (int i = 0, n = this_._internal_string_val_size(); i < n; ++i) {
    const auto& s = this_._internal_string_val().Get(i);
    target = stream->WriteBytes(8, s, target);
  }

  // repeated float scomplex_val = 9 [packed = true];
  if (this_._internal_scomplex_val_size() > 0) {
    target = stream->WriteFixedPacked(9, this_._internal_scomplex_val(), target);
  }

  // repeated int64 int64_val = 10 [packed = true];
  {
    int byte_size = this_._impl_._int64_val_cached_byte_size_.Get();
    if (byte_size > 0) {
      target = stream->WriteInt64Packed(
          10, this_._internal_int64_val(), byte_size, target);
    }
  }

  // repeated bool bool_val = 11 [packed = true];
  if (this_._internal_bool_val_size() > 0) {
    target = stream->WriteFixedPacked(11, this_._internal_bool_val(), target);
  }

  // repeated double dcomplex_val = 12 [packed = true];
  if (this_._internal_dcomplex_val_size() > 0) {
    target = stream->WriteFixedPacked(12, this_._internal_dcomplex_val(), target);
  }

  // repeated int32 half_val = 13 [packed = true];
  {
    int byte_size = this_._impl_._half_val_cached_byte_size_.Get();
    if (byte_size > 0) {
      target = stream->WriteInt32Packed(
          13, this_._internal_half_val(), byte_size, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:opencv_tensorflow.TensorProto)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t TensorProto::ByteSizeLong(const MessageLite& base) {
  const TensorProto& this_ = static_cast<const TensorProto&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t TensorProto::ByteSizeLong() const {
  const TensorProto& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:opencv_tensorflow.TensorProto)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
   {
    // repeated float float_val = 5 [packed = true];
    {
      ::size_t data_size = ::size_t{4} *
          ::_pbi::FromIntSize(this_._internal_float_val_size());
      ::size_t tag_size = data_size == 0
          ? 0
          : 1 + ::_pbi::WireFormatLite::Int32Size(
                              static_cast<::int32_t>(data_size));
      total_size += tag_size + data_size;
    }
    // repeated double double_val = 6 [packed = true];
    {
      ::size_t data_size = ::size_t{8} *
          ::_pbi::FromIntSize(this_._internal_double_val_size());
      ::size_t tag_size = data_size == 0
          ? 0
          : 1 + ::_pbi::WireFormatLite::Int32Size(
                              static_cast<::int32_t>(data_size));
      total_size += tag_size + data_size;
    }
    // repeated int32 int_val = 7 [packed = true];
    {
      total_size +=
          ::_pbi::WireFormatLite::Int32SizeWithPackedTagSize(
              this_._internal_int_val(), 1,
              this_._impl_._int_val_cached_byte_size_);
    }
    // repeated bytes string_val = 8;
    {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_string_val().size());
      for (int i = 0, n = this_._internal_string_val().size(); i < n; ++i) {
        total_size += ::google::protobuf::internal::WireFormatLite::BytesSize(
            this_._internal_string_val().Get(i));
      }
    }
    // repeated float scomplex_val = 9 [packed = true];
    {
      ::size_t data_size = ::size_t{4} *
          ::_pbi::FromIntSize(this_._internal_scomplex_val_size());
      ::size_t tag_size = data_size == 0
          ? 0
          : 1 + ::_pbi::WireFormatLite::Int32Size(
                              static_cast<::int32_t>(data_size));
      total_size += tag_size + data_size;
    }
    // repeated int64 int64_val = 10 [packed = true];
    {
      total_size +=
          ::_pbi::WireFormatLite::Int64SizeWithPackedTagSize(
              this_._internal_int64_val(), 1,
              this_._impl_._int64_val_cached_byte_size_);
    }
    // repeated bool bool_val = 11 [packed = true];
    {
      ::size_t data_size = ::size_t{1} *
          ::_pbi::FromIntSize(this_._internal_bool_val_size());
      ::size_t tag_size = data_size == 0
          ? 0
          : 1 + ::_pbi::WireFormatLite::Int32Size(
                              static_cast<::int32_t>(data_size));
      total_size += tag_size + data_size;
    }
    // repeated double dcomplex_val = 12 [packed = true];
    {
      ::size_t data_size = ::size_t{8} *
          ::_pbi::FromIntSize(this_._internal_dcomplex_val_size());
      ::size_t tag_size = data_size == 0
          ? 0
          : 1 + ::_pbi::WireFormatLite::Int32Size(
                              static_cast<::int32_t>(data_size));
      total_size += tag_size + data_size;
    }
    // repeated int32 half_val = 13 [packed = true];
    {
      total_size +=
          ::_pbi::WireFormatLite::Int32SizeWithPackedTagSize(
              this_._internal_half_val(), 1,
              this_._impl_._half_val_cached_byte_size_);
    }
  }
  cached_has_bits = this_._impl_._has_bits_[0];
  if ((cached_has_bits & 0x0000000fU) != 0) {
    // bytes tensor_content = 4;
    if ((cached_has_bits & 0x00000001U) != 0) {
      if (!this_._internal_tensor_content().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::BytesSize(
                                        this_._internal_tensor_content());
      }
    }
    // .opencv_tensorflow.TensorShapeProto tensor_shape = 2;
    if ((cached_has_bits & 0x00000002U) != 0) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.tensor_shape_);
    }
    // .opencv_tensorflow.DataType dtype = 1;
    if ((cached_has_bits & 0x00000004U) != 0) {
      if (this_._internal_dtype() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_dtype());
      }
    }
    // int32 version_number = 3;
    if ((cached_has_bits & 0x00000008U) != 0) {
      if (this_._internal_version_number() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_version_number());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void TensorProto::MergeImpl(::google::protobuf::MessageLite& to_msg, const ::google::protobuf::MessageLite& from_msg) {
  auto* const _this = static_cast<TensorProto*>(&to_msg);
  auto& from = static_cast<const TensorProto&>(from_msg);
  if constexpr (::_pbi::DebugHardenVerifyHasBitConsistency()) {
    from.VerifyHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:opencv_tensorflow.TensorProto)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  _this->_internal_mutable_float_val()->MergeFrom(from._internal_float_val());
  _this->_internal_mutable_double_val()->MergeFrom(from._internal_double_val());
  _this->_internal_mutable_int_val()->MergeFrom(from._internal_int_val());
  _this->_internal_mutable_string_val()->MergeFrom(from._internal_string_val());
  _this->_internal_mutable_scomplex_val()->MergeFrom(from._internal_scomplex_val());
  _this->_internal_mutable_int64_val()->MergeFrom(from._internal_int64_val());
  _this->_internal_mutable_bool_val()->MergeFrom(from._internal_bool_val());
  _this->_internal_mutable_dcomplex_val()->MergeFrom(from._internal_dcomplex_val());
  _this->_internal_mutable_half_val()->MergeFrom(from._internal_half_val());
  cached_has_bits = from._impl_._has_bits_[0];
  if ((cached_has_bits & 0x0000000fU) != 0) {
    if ((cached_has_bits & 0x00000001U) != 0) {
      if (!from._internal_tensor_content().empty()) {
        _this->_internal_set_tensor_content(from._internal_tensor_content());
      } else {
        if (_this->_impl_.tensor_content_.IsDefault()) {
          _this->_internal_set_tensor_content("");
        }
      }
    }
    if ((cached_has_bits & 0x00000002U) != 0) {
      ABSL_DCHECK(from._impl_.tensor_shape_ != nullptr);
      if (_this->_impl_.tensor_shape_ == nullptr) {
        _this->_impl_.tensor_shape_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.tensor_shape_);
      } else {
        _this->_impl_.tensor_shape_->MergeFrom(*from._impl_.tensor_shape_);
      }
    }
    if ((cached_has_bits & 0x00000004U) != 0) {
      if (from._internal_dtype() != 0) {
        _this->_impl_.dtype_ = from._impl_.dtype_;
      }
    }
    if ((cached_has_bits & 0x00000008U) != 0) {
      if (from._internal_version_number() != 0) {
        _this->_impl_.version_number_ = from._impl_.version_number_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(from._internal_metadata_);
}

void TensorProto::CopyFrom(const TensorProto& from) {
// @@protoc_insertion_point(class_specific_copy_from_start:opencv_tensorflow.TensorProto)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void TensorProto::InternalSwap(TensorProto* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.float_val_.InternalSwap(&other->_impl_.float_val_);
  _impl_.double_val_.InternalSwap(&other->_impl_.double_val_);
  _impl_.int_val_.InternalSwap(&other->_impl_.int_val_);
  _impl_.string_val_.InternalSwap(&other->_impl_.string_val_);
  _impl_.scomplex_val_.InternalSwap(&other->_impl_.scomplex_val_);
  _impl_.int64_val_.InternalSwap(&other->_impl_.int64_val_);
  _impl_.bool_val_.InternalSwap(&other->_impl_.bool_val_);
  _impl_.dcomplex_val_.InternalSwap(&other->_impl_.dcomplex_val_);
  _impl_.half_val_.InternalSwap(&other->_impl_.half_val_);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.tensor_content_, &other->_impl_.tensor_content_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.version_number_)
      + sizeof(TensorProto::_impl_.version_number_)
      - PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.tensor_shape_)>(
          reinterpret_cast<char*>(&_impl_.tensor_shape_),
          reinterpret_cast<char*>(&other->_impl_.tensor_shape_));
}

::google::protobuf::Metadata TensorProto::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// @@protoc_insertion_point(namespace_scope)
}  // namespace opencv_tensorflow
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google
// @@protoc_insertion_point(global_scope)
PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 static ::std::false_type
    _static_init2_ [[maybe_unused]] =
        (::_pbi::AddDescriptors(&descriptor_table_tensor_2eproto),
         ::std::false_type{});
#include "google/protobuf/port_undef.inc"
