// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: function.proto
// Protobuf C++ Version: 6.32.1

#ifndef function_2eproto_2epb_2eh
#define function_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 6032001
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/message.h"
#include "google/protobuf/message_lite.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/map.h"  // IWYU pragma: export
#include "google/protobuf/map_type_handler.h"  // IWYU pragma: export
#include "google/protobuf/map_entry.h"
#include "google/protobuf/map_field.h"
#include "google/protobuf/unknown_field_set.h"
#include "attr_value.pb.h"
#include "op_def.pb.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_function_2eproto

namespace google {
namespace protobuf {
namespace internal {
template <typename T>
::absl::string_view GetAnyMessageName();
}  // namespace internal
}  // namespace protobuf
}  // namespace google

// Internal implementation detail -- do not use these members.
struct TableStruct_function_2eproto {
  static const ::uint32_t offsets[];
};
extern "C" {
extern const ::google::protobuf::internal::DescriptorTable descriptor_table_function_2eproto;
}  // extern "C"
namespace opencv_tensorflow {
class FunctionDef;
struct FunctionDefDefaultTypeInternal;
extern FunctionDefDefaultTypeInternal _FunctionDef_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull FunctionDef_class_data_;
class FunctionDefLibrary;
struct FunctionDefLibraryDefaultTypeInternal;
extern FunctionDefLibraryDefaultTypeInternal _FunctionDefLibrary_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull FunctionDefLibrary_class_data_;
class FunctionDef_Node;
struct FunctionDef_NodeDefaultTypeInternal;
extern FunctionDef_NodeDefaultTypeInternal _FunctionDef_Node_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull FunctionDef_Node_class_data_;
class FunctionDef_Node_AttrEntry_DoNotUse;
struct FunctionDef_Node_AttrEntry_DoNotUseDefaultTypeInternal;
extern FunctionDef_Node_AttrEntry_DoNotUseDefaultTypeInternal _FunctionDef_Node_AttrEntry_DoNotUse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull FunctionDef_Node_AttrEntry_DoNotUse_class_data_;
class GradientDef;
struct GradientDefDefaultTypeInternal;
extern GradientDefDefaultTypeInternal _GradientDef_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull GradientDef_class_data_;
}  // namespace opencv_tensorflow
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google

namespace opencv_tensorflow {

// ===================================================================


// -------------------------------------------------------------------

class GradientDef final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:opencv_tensorflow.GradientDef) */ {
 public:
  inline GradientDef() : GradientDef(nullptr) {}
  ~GradientDef() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(GradientDef* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(GradientDef));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR GradientDef(::google::protobuf::internal::ConstantInitialized);

  inline GradientDef(const GradientDef& from) : GradientDef(nullptr, from) {}
  inline GradientDef(GradientDef&& from) noexcept
      : GradientDef(nullptr, ::std::move(from)) {}
  inline GradientDef& operator=(const GradientDef& from) {
    CopyFrom(from);
    return *this;
  }
  inline GradientDef& operator=(GradientDef&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const GradientDef& default_instance() {
    return *reinterpret_cast<const GradientDef*>(
        &_GradientDef_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 4;
  friend void swap(GradientDef& a, GradientDef& b) { a.Swap(&b); }
  inline void Swap(GradientDef* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(GradientDef* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  GradientDef* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<GradientDef>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const GradientDef& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const GradientDef& from) { GradientDef::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(GradientDef* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "opencv_tensorflow.GradientDef"; }

 protected:
  explicit GradientDef(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  GradientDef(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const GradientDef& from);
  GradientDef(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, GradientDef&& from) noexcept
      : GradientDef(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kFunctionNameFieldNumber = 1,
    kGradientFuncFieldNumber = 2,
  };
  // string function_name = 1;
  void clear_function_name() ;
  const ::std::string& function_name() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_function_name(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_function_name();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_function_name();
  void set_allocated_function_name(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_function_name() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_function_name(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_function_name();

  public:
  // string gradient_func = 2;
  void clear_gradient_func() ;
  const ::std::string& gradient_func() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_gradient_func(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_gradient_func();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_gradient_func();
  void set_allocated_gradient_func(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_gradient_func() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_gradient_func(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_gradient_func();

  public:
  // @@protoc_insertion_point(class_scope:opencv_tensorflow.GradientDef)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   0, 64,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const GradientDef& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr function_name_;
    ::google::protobuf::internal::ArenaStringPtr gradient_func_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_function_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull GradientDef_class_data_;
// -------------------------------------------------------------------

class FunctionDef_Node_AttrEntry_DoNotUse final
    : public ::google::protobuf::internal::MapEntry<::std::string, ::google::protobuf::Message,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                             ::google::protobuf::internal::WireFormatLite::TYPE_MESSAGE> {
 public:
  using SuperType =
      ::google::protobuf::internal::MapEntry<::std::string, ::google::protobuf::Message,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_MESSAGE>;
  FunctionDef_Node_AttrEntry_DoNotUse();
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR FunctionDef_Node_AttrEntry_DoNotUse(::google::protobuf::internal::ConstantInitialized);
  explicit FunctionDef_Node_AttrEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr const void* PROTOBUF_NONNULL internal_default_instance() {
    return &_FunctionDef_Node_AttrEntry_DoNotUse_default_instance_;
  }


  static constexpr auto InternalGenerateClassData_();

 private:
  friend class ::google::protobuf::MessageLite;
  friend struct ::TableStruct_function_2eproto;

  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   1, 56,
                                   2>
      _table_;

  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();
};
extern const ::google::protobuf::internal::ClassDataFull FunctionDef_Node_AttrEntry_DoNotUse_class_data_;
// -------------------------------------------------------------------

class FunctionDef_Node final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:opencv_tensorflow.FunctionDef.Node) */ {
 public:
  inline FunctionDef_Node() : FunctionDef_Node(nullptr) {}
  ~FunctionDef_Node() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(FunctionDef_Node* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(FunctionDef_Node));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR FunctionDef_Node(::google::protobuf::internal::ConstantInitialized);

  inline FunctionDef_Node(const FunctionDef_Node& from) : FunctionDef_Node(nullptr, from) {}
  inline FunctionDef_Node(FunctionDef_Node&& from) noexcept
      : FunctionDef_Node(nullptr, ::std::move(from)) {}
  inline FunctionDef_Node& operator=(const FunctionDef_Node& from) {
    CopyFrom(from);
    return *this;
  }
  inline FunctionDef_Node& operator=(FunctionDef_Node&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const FunctionDef_Node& default_instance() {
    return *reinterpret_cast<const FunctionDef_Node*>(
        &_FunctionDef_Node_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 2;
  friend void swap(FunctionDef_Node& a, FunctionDef_Node& b) { a.Swap(&b); }
  inline void Swap(FunctionDef_Node* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(FunctionDef_Node* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  FunctionDef_Node* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<FunctionDef_Node>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const FunctionDef_Node& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const FunctionDef_Node& from) { FunctionDef_Node::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(FunctionDef_Node* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "opencv_tensorflow.FunctionDef.Node"; }

 protected:
  explicit FunctionDef_Node(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  FunctionDef_Node(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const FunctionDef_Node& from);
  FunctionDef_Node(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, FunctionDef_Node&& from) noexcept
      : FunctionDef_Node(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kRetFieldNumber = 1,
    kArgFieldNumber = 3,
    kDepFieldNumber = 4,
    kAttrFieldNumber = 5,
    kOpFieldNumber = 2,
  };
  // repeated string ret = 1;
  int ret_size() const;
  private:
  int _internal_ret_size() const;

  public:
  void clear_ret() ;
  const ::std::string& ret(int index) const;
  ::std::string* PROTOBUF_NONNULL mutable_ret(int index);
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_ret(int index, Arg_&& value, Args_... args);
  ::std::string* PROTOBUF_NONNULL add_ret();
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void add_ret(Arg_&& value, Args_... args);
  const ::google::protobuf::RepeatedPtrField<::std::string>& ret() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL mutable_ret();

  private:
  const ::google::protobuf::RepeatedPtrField<::std::string>& _internal_ret() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL _internal_mutable_ret();

  public:
  // repeated string arg = 3;
  int arg_size() const;
  private:
  int _internal_arg_size() const;

  public:
  void clear_arg() ;
  const ::std::string& arg(int index) const;
  ::std::string* PROTOBUF_NONNULL mutable_arg(int index);
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_arg(int index, Arg_&& value, Args_... args);
  ::std::string* PROTOBUF_NONNULL add_arg();
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void add_arg(Arg_&& value, Args_... args);
  const ::google::protobuf::RepeatedPtrField<::std::string>& arg() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL mutable_arg();

  private:
  const ::google::protobuf::RepeatedPtrField<::std::string>& _internal_arg() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL _internal_mutable_arg();

  public:
  // repeated string dep = 4;
  int dep_size() const;
  private:
  int _internal_dep_size() const;

  public:
  void clear_dep() ;
  const ::std::string& dep(int index) const;
  ::std::string* PROTOBUF_NONNULL mutable_dep(int index);
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_dep(int index, Arg_&& value, Args_... args);
  ::std::string* PROTOBUF_NONNULL add_dep();
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void add_dep(Arg_&& value, Args_... args);
  const ::google::protobuf::RepeatedPtrField<::std::string>& dep() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL mutable_dep();

  private:
  const ::google::protobuf::RepeatedPtrField<::std::string>& _internal_dep() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL _internal_mutable_dep();

  public:
  // map<string, .opencv_tensorflow.AttrValue> attr = 5;
  int attr_size() const;
  private:
  int _internal_attr_size() const;

  public:
  void clear_attr() ;
  const ::google::protobuf::Map<::std::string, ::opencv_tensorflow::AttrValue>& attr() const;
  ::google::protobuf::Map<::std::string, ::opencv_tensorflow::AttrValue>* PROTOBUF_NONNULL mutable_attr();

  private:
  const ::google::protobuf::Map<::std::string, ::opencv_tensorflow::AttrValue>& _internal_attr() const;
  ::google::protobuf::Map<::std::string, ::opencv_tensorflow::AttrValue>* PROTOBUF_NONNULL _internal_mutable_attr();

  public:
  // string op = 2;
  void clear_op() ;
  const ::std::string& op() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_op(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_op();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_op();
  void set_allocated_op(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_op() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_op(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_op();

  public:
  // @@protoc_insertion_point(class_scope:opencv_tensorflow.FunctionDef.Node)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<2, 5,
                                   2, 58,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const FunctionDef_Node& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField<::std::string> ret_;
    ::google::protobuf::RepeatedPtrField<::std::string> arg_;
    ::google::protobuf::RepeatedPtrField<::std::string> dep_;
    ::google::protobuf::internal::MapField<FunctionDef_Node_AttrEntry_DoNotUse, ::std::string, ::opencv_tensorflow::AttrValue,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_MESSAGE>
        attr_;
    ::google::protobuf::internal::ArenaStringPtr op_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_function_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull FunctionDef_Node_class_data_;
// -------------------------------------------------------------------

class FunctionDef final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:opencv_tensorflow.FunctionDef) */ {
 public:
  inline FunctionDef() : FunctionDef(nullptr) {}
  ~FunctionDef() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(FunctionDef* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(FunctionDef));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR FunctionDef(::google::protobuf::internal::ConstantInitialized);

  inline FunctionDef(const FunctionDef& from) : FunctionDef(nullptr, from) {}
  inline FunctionDef(FunctionDef&& from) noexcept
      : FunctionDef(nullptr, ::std::move(from)) {}
  inline FunctionDef& operator=(const FunctionDef& from) {
    CopyFrom(from);
    return *this;
  }
  inline FunctionDef& operator=(FunctionDef&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const FunctionDef& default_instance() {
    return *reinterpret_cast<const FunctionDef*>(
        &_FunctionDef_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 3;
  friend void swap(FunctionDef& a, FunctionDef& b) { a.Swap(&b); }
  inline void Swap(FunctionDef* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(FunctionDef* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  FunctionDef* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<FunctionDef>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const FunctionDef& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const FunctionDef& from) { FunctionDef::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(FunctionDef* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "opencv_tensorflow.FunctionDef"; }

 protected:
  explicit FunctionDef(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  FunctionDef(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const FunctionDef& from);
  FunctionDef(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, FunctionDef&& from) noexcept
      : FunctionDef(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------
  using Node = FunctionDef_Node;

  // accessors -------------------------------------------------------
  enum : int {
    kNodeFieldNumber = 2,
    kSignatureFieldNumber = 1,
  };
  // repeated .opencv_tensorflow.FunctionDef.Node node = 2;
  int node_size() const;
  private:
  int _internal_node_size() const;

  public:
  void clear_node() ;
  ::opencv_tensorflow::FunctionDef_Node* PROTOBUF_NONNULL mutable_node(int index);
  ::google::protobuf::RepeatedPtrField<::opencv_tensorflow::FunctionDef_Node>* PROTOBUF_NONNULL mutable_node();

  private:
  const ::google::protobuf::RepeatedPtrField<::opencv_tensorflow::FunctionDef_Node>& _internal_node() const;
  ::google::protobuf::RepeatedPtrField<::opencv_tensorflow::FunctionDef_Node>* PROTOBUF_NONNULL _internal_mutable_node();
  public:
  const ::opencv_tensorflow::FunctionDef_Node& node(int index) const;
  ::opencv_tensorflow::FunctionDef_Node* PROTOBUF_NONNULL add_node();
  const ::google::protobuf::RepeatedPtrField<::opencv_tensorflow::FunctionDef_Node>& node() const;
  // .opencv_tensorflow.OpDef signature = 1;
  bool has_signature() const;
  void clear_signature() ;
  const ::opencv_tensorflow::OpDef& signature() const;
  [[nodiscard]] ::opencv_tensorflow::OpDef* PROTOBUF_NULLABLE release_signature();
  ::opencv_tensorflow::OpDef* PROTOBUF_NONNULL mutable_signature();
  void set_allocated_signature(::opencv_tensorflow::OpDef* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_signature(::opencv_tensorflow::OpDef* PROTOBUF_NULLABLE value);
  ::opencv_tensorflow::OpDef* PROTOBUF_NULLABLE unsafe_arena_release_signature();

  private:
  const ::opencv_tensorflow::OpDef& _internal_signature() const;
  ::opencv_tensorflow::OpDef* PROTOBUF_NONNULL _internal_mutable_signature();

  public:
  // @@protoc_insertion_point(class_scope:opencv_tensorflow.FunctionDef)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   2, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const FunctionDef& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::opencv_tensorflow::FunctionDef_Node > node_;
    ::opencv_tensorflow::OpDef* PROTOBUF_NULLABLE signature_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_function_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull FunctionDef_class_data_;
// -------------------------------------------------------------------

class FunctionDefLibrary final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:opencv_tensorflow.FunctionDefLibrary) */ {
 public:
  inline FunctionDefLibrary() : FunctionDefLibrary(nullptr) {}
  ~FunctionDefLibrary() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(FunctionDefLibrary* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(FunctionDefLibrary));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR FunctionDefLibrary(::google::protobuf::internal::ConstantInitialized);

  inline FunctionDefLibrary(const FunctionDefLibrary& from) : FunctionDefLibrary(nullptr, from) {}
  inline FunctionDefLibrary(FunctionDefLibrary&& from) noexcept
      : FunctionDefLibrary(nullptr, ::std::move(from)) {}
  inline FunctionDefLibrary& operator=(const FunctionDefLibrary& from) {
    CopyFrom(from);
    return *this;
  }
  inline FunctionDefLibrary& operator=(FunctionDefLibrary&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const FunctionDefLibrary& default_instance() {
    return *reinterpret_cast<const FunctionDefLibrary*>(
        &_FunctionDefLibrary_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(FunctionDefLibrary& a, FunctionDefLibrary& b) { a.Swap(&b); }
  inline void Swap(FunctionDefLibrary* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(FunctionDefLibrary* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  FunctionDefLibrary* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<FunctionDefLibrary>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const FunctionDefLibrary& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const FunctionDefLibrary& from) { FunctionDefLibrary::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(FunctionDefLibrary* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "opencv_tensorflow.FunctionDefLibrary"; }

 protected:
  explicit FunctionDefLibrary(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  FunctionDefLibrary(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const FunctionDefLibrary& from);
  FunctionDefLibrary(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, FunctionDefLibrary&& from) noexcept
      : FunctionDefLibrary(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kFunctionFieldNumber = 1,
    kGradientFieldNumber = 2,
  };
  // repeated .opencv_tensorflow.FunctionDef function = 1;
  int function_size() const;
  private:
  int _internal_function_size() const;

  public:
  void clear_function() ;
  ::opencv_tensorflow::FunctionDef* PROTOBUF_NONNULL mutable_function(int index);
  ::google::protobuf::RepeatedPtrField<::opencv_tensorflow::FunctionDef>* PROTOBUF_NONNULL mutable_function();

  private:
  const ::google::protobuf::RepeatedPtrField<::opencv_tensorflow::FunctionDef>& _internal_function() const;
  ::google::protobuf::RepeatedPtrField<::opencv_tensorflow::FunctionDef>* PROTOBUF_NONNULL _internal_mutable_function();
  public:
  const ::opencv_tensorflow::FunctionDef& function(int index) const;
  ::opencv_tensorflow::FunctionDef* PROTOBUF_NONNULL add_function();
  const ::google::protobuf::RepeatedPtrField<::opencv_tensorflow::FunctionDef>& function() const;
  // repeated .opencv_tensorflow.GradientDef gradient = 2;
  int gradient_size() const;
  private:
  int _internal_gradient_size() const;

  public:
  void clear_gradient() ;
  ::opencv_tensorflow::GradientDef* PROTOBUF_NONNULL mutable_gradient(int index);
  ::google::protobuf::RepeatedPtrField<::opencv_tensorflow::GradientDef>* PROTOBUF_NONNULL mutable_gradient();

  private:
  const ::google::protobuf::RepeatedPtrField<::opencv_tensorflow::GradientDef>& _internal_gradient() const;
  ::google::protobuf::RepeatedPtrField<::opencv_tensorflow::GradientDef>* PROTOBUF_NONNULL _internal_mutable_gradient();
  public:
  const ::opencv_tensorflow::GradientDef& gradient(int index) const;
  ::opencv_tensorflow::GradientDef* PROTOBUF_NONNULL add_gradient();
  const ::google::protobuf::RepeatedPtrField<::opencv_tensorflow::GradientDef>& gradient() const;
  // @@protoc_insertion_point(class_scope:opencv_tensorflow.FunctionDefLibrary)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   2, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const FunctionDefLibrary& from_msg);
    ::google::protobuf::RepeatedPtrField< ::opencv_tensorflow::FunctionDef > function_;
    ::google::protobuf::RepeatedPtrField< ::opencv_tensorflow::GradientDef > gradient_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_function_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull FunctionDefLibrary_class_data_;

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// FunctionDefLibrary

// repeated .opencv_tensorflow.FunctionDef function = 1;
inline int FunctionDefLibrary::_internal_function_size() const {
  return _internal_function().size();
}
inline int FunctionDefLibrary::function_size() const {
  return _internal_function_size();
}
inline void FunctionDefLibrary::clear_function() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.function_.Clear();
}
inline ::opencv_tensorflow::FunctionDef* PROTOBUF_NONNULL FunctionDefLibrary::mutable_function(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:opencv_tensorflow.FunctionDefLibrary.function)
  return _internal_mutable_function()->Mutable(index);
}
inline ::google::protobuf::RepeatedPtrField<::opencv_tensorflow::FunctionDef>* PROTOBUF_NONNULL FunctionDefLibrary::mutable_function()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable_list:opencv_tensorflow.FunctionDefLibrary.function)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_function();
}
inline const ::opencv_tensorflow::FunctionDef& FunctionDefLibrary::function(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opencv_tensorflow.FunctionDefLibrary.function)
  return _internal_function().Get(index);
}
inline ::opencv_tensorflow::FunctionDef* PROTOBUF_NONNULL FunctionDefLibrary::add_function()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::opencv_tensorflow::FunctionDef* _add = _internal_mutable_function()->Add();
  // @@protoc_insertion_point(field_add:opencv_tensorflow.FunctionDefLibrary.function)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::opencv_tensorflow::FunctionDef>& FunctionDefLibrary::function() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opencv_tensorflow.FunctionDefLibrary.function)
  return _internal_function();
}
inline const ::google::protobuf::RepeatedPtrField<::opencv_tensorflow::FunctionDef>&
FunctionDefLibrary::_internal_function() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.function_;
}
inline ::google::protobuf::RepeatedPtrField<::opencv_tensorflow::FunctionDef>* PROTOBUF_NONNULL
FunctionDefLibrary::_internal_mutable_function() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.function_;
}

// repeated .opencv_tensorflow.GradientDef gradient = 2;
inline int FunctionDefLibrary::_internal_gradient_size() const {
  return _internal_gradient().size();
}
inline int FunctionDefLibrary::gradient_size() const {
  return _internal_gradient_size();
}
inline void FunctionDefLibrary::clear_gradient() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.gradient_.Clear();
}
inline ::opencv_tensorflow::GradientDef* PROTOBUF_NONNULL FunctionDefLibrary::mutable_gradient(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:opencv_tensorflow.FunctionDefLibrary.gradient)
  return _internal_mutable_gradient()->Mutable(index);
}
inline ::google::protobuf::RepeatedPtrField<::opencv_tensorflow::GradientDef>* PROTOBUF_NONNULL FunctionDefLibrary::mutable_gradient()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable_list:opencv_tensorflow.FunctionDefLibrary.gradient)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_gradient();
}
inline const ::opencv_tensorflow::GradientDef& FunctionDefLibrary::gradient(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opencv_tensorflow.FunctionDefLibrary.gradient)
  return _internal_gradient().Get(index);
}
inline ::opencv_tensorflow::GradientDef* PROTOBUF_NONNULL FunctionDefLibrary::add_gradient()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::opencv_tensorflow::GradientDef* _add = _internal_mutable_gradient()->Add();
  // @@protoc_insertion_point(field_add:opencv_tensorflow.FunctionDefLibrary.gradient)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::opencv_tensorflow::GradientDef>& FunctionDefLibrary::gradient() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opencv_tensorflow.FunctionDefLibrary.gradient)
  return _internal_gradient();
}
inline const ::google::protobuf::RepeatedPtrField<::opencv_tensorflow::GradientDef>&
FunctionDefLibrary::_internal_gradient() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.gradient_;
}
inline ::google::protobuf::RepeatedPtrField<::opencv_tensorflow::GradientDef>* PROTOBUF_NONNULL
FunctionDefLibrary::_internal_mutable_gradient() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.gradient_;
}

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// FunctionDef_Node

// repeated string ret = 1;
inline int FunctionDef_Node::_internal_ret_size() const {
  return _internal_ret().size();
}
inline int FunctionDef_Node::ret_size() const {
  return _internal_ret_size();
}
inline void FunctionDef_Node::clear_ret() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.ret_.Clear();
}
inline ::std::string* PROTOBUF_NONNULL FunctionDef_Node::add_ret()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::std::string* _s = _internal_mutable_ret()->Add();
  // @@protoc_insertion_point(field_add_mutable:opencv_tensorflow.FunctionDef.Node.ret)
  return _s;
}
inline const ::std::string& FunctionDef_Node::ret(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opencv_tensorflow.FunctionDef.Node.ret)
  return _internal_ret().Get(index);
}
inline ::std::string* PROTOBUF_NONNULL FunctionDef_Node::mutable_ret(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:opencv_tensorflow.FunctionDef.Node.ret)
  return _internal_mutable_ret()->Mutable(index);
}
template <typename Arg_, typename... Args_>
inline void FunctionDef_Node::set_ret(int index, Arg_&& value, Args_... args) {
  ::google::protobuf::internal::AssignToString(*_internal_mutable_ret()->Mutable(index), ::std::forward<Arg_>(value),
                        args... );
  // @@protoc_insertion_point(field_set:opencv_tensorflow.FunctionDef.Node.ret)
}
template <typename Arg_, typename... Args_>
inline void FunctionDef_Node::add_ret(Arg_&& value, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::google::protobuf::internal::AddToRepeatedPtrField(*_internal_mutable_ret(),
                               ::std::forward<Arg_>(value),
                               args... );
  // @@protoc_insertion_point(field_add:opencv_tensorflow.FunctionDef.Node.ret)
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>& FunctionDef_Node::ret()
    const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opencv_tensorflow.FunctionDef.Node.ret)
  return _internal_ret();
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
FunctionDef_Node::mutable_ret() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable_list:opencv_tensorflow.FunctionDef.Node.ret)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_ret();
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>&
FunctionDef_Node::_internal_ret() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.ret_;
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
FunctionDef_Node::_internal_mutable_ret() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.ret_;
}

// string op = 2;
inline void FunctionDef_Node::clear_op() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.op_.ClearToEmpty();
  _impl_._has_bits_[0] &= ~0x00000001U;
}
inline const ::std::string& FunctionDef_Node::op() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opencv_tensorflow.FunctionDef.Node.op)
  return _internal_op();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void FunctionDef_Node::set_op(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_._has_bits_[0] |= 0x00000001U;
  _impl_.op_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:opencv_tensorflow.FunctionDef.Node.op)
}
inline ::std::string* PROTOBUF_NONNULL FunctionDef_Node::mutable_op()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::std::string* _s = _internal_mutable_op();
  // @@protoc_insertion_point(field_mutable:opencv_tensorflow.FunctionDef.Node.op)
  return _s;
}
inline const ::std::string& FunctionDef_Node::_internal_op() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.op_.Get();
}
inline void FunctionDef_Node::_internal_set_op(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_._has_bits_[0] |= 0x00000001U;
  _impl_.op_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL FunctionDef_Node::_internal_mutable_op() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_._has_bits_[0] |= 0x00000001U;
  return _impl_.op_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE FunctionDef_Node::release_op() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:opencv_tensorflow.FunctionDef.Node.op)
  if ((_impl_._has_bits_[0] & 0x00000001U) == 0) {
    return nullptr;
  }
  _impl_._has_bits_[0] &= ~0x00000001U;
  auto* released = _impl_.op_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.op_.Set("", GetArena());
  }
  return released;
}
inline void FunctionDef_Node::set_allocated_op(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    _impl_._has_bits_[0] |= 0x00000001U;
  } else {
    _impl_._has_bits_[0] &= ~0x00000001U;
  }
  _impl_.op_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.op_.IsDefault()) {
    _impl_.op_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:opencv_tensorflow.FunctionDef.Node.op)
}

// repeated string arg = 3;
inline int FunctionDef_Node::_internal_arg_size() const {
  return _internal_arg().size();
}
inline int FunctionDef_Node::arg_size() const {
  return _internal_arg_size();
}
inline void FunctionDef_Node::clear_arg() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.arg_.Clear();
}
inline ::std::string* PROTOBUF_NONNULL FunctionDef_Node::add_arg()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::std::string* _s = _internal_mutable_arg()->Add();
  // @@protoc_insertion_point(field_add_mutable:opencv_tensorflow.FunctionDef.Node.arg)
  return _s;
}
inline const ::std::string& FunctionDef_Node::arg(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opencv_tensorflow.FunctionDef.Node.arg)
  return _internal_arg().Get(index);
}
inline ::std::string* PROTOBUF_NONNULL FunctionDef_Node::mutable_arg(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:opencv_tensorflow.FunctionDef.Node.arg)
  return _internal_mutable_arg()->Mutable(index);
}
template <typename Arg_, typename... Args_>
inline void FunctionDef_Node::set_arg(int index, Arg_&& value, Args_... args) {
  ::google::protobuf::internal::AssignToString(*_internal_mutable_arg()->Mutable(index), ::std::forward<Arg_>(value),
                        args... );
  // @@protoc_insertion_point(field_set:opencv_tensorflow.FunctionDef.Node.arg)
}
template <typename Arg_, typename... Args_>
inline void FunctionDef_Node::add_arg(Arg_&& value, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::google::protobuf::internal::AddToRepeatedPtrField(*_internal_mutable_arg(),
                               ::std::forward<Arg_>(value),
                               args... );
  // @@protoc_insertion_point(field_add:opencv_tensorflow.FunctionDef.Node.arg)
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>& FunctionDef_Node::arg()
    const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opencv_tensorflow.FunctionDef.Node.arg)
  return _internal_arg();
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
FunctionDef_Node::mutable_arg() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable_list:opencv_tensorflow.FunctionDef.Node.arg)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_arg();
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>&
FunctionDef_Node::_internal_arg() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.arg_;
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
FunctionDef_Node::_internal_mutable_arg() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.arg_;
}

// repeated string dep = 4;
inline int FunctionDef_Node::_internal_dep_size() const {
  return _internal_dep().size();
}
inline int FunctionDef_Node::dep_size() const {
  return _internal_dep_size();
}
inline void FunctionDef_Node::clear_dep() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.dep_.Clear();
}
inline ::std::string* PROTOBUF_NONNULL FunctionDef_Node::add_dep()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::std::string* _s = _internal_mutable_dep()->Add();
  // @@protoc_insertion_point(field_add_mutable:opencv_tensorflow.FunctionDef.Node.dep)
  return _s;
}
inline const ::std::string& FunctionDef_Node::dep(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opencv_tensorflow.FunctionDef.Node.dep)
  return _internal_dep().Get(index);
}
inline ::std::string* PROTOBUF_NONNULL FunctionDef_Node::mutable_dep(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:opencv_tensorflow.FunctionDef.Node.dep)
  return _internal_mutable_dep()->Mutable(index);
}
template <typename Arg_, typename... Args_>
inline void FunctionDef_Node::set_dep(int index, Arg_&& value, Args_... args) {
  ::google::protobuf::internal::AssignToString(*_internal_mutable_dep()->Mutable(index), ::std::forward<Arg_>(value),
                        args... );
  // @@protoc_insertion_point(field_set:opencv_tensorflow.FunctionDef.Node.dep)
}
template <typename Arg_, typename... Args_>
inline void FunctionDef_Node::add_dep(Arg_&& value, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::google::protobuf::internal::AddToRepeatedPtrField(*_internal_mutable_dep(),
                               ::std::forward<Arg_>(value),
                               args... );
  // @@protoc_insertion_point(field_add:opencv_tensorflow.FunctionDef.Node.dep)
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>& FunctionDef_Node::dep()
    const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opencv_tensorflow.FunctionDef.Node.dep)
  return _internal_dep();
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
FunctionDef_Node::mutable_dep() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable_list:opencv_tensorflow.FunctionDef.Node.dep)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_dep();
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>&
FunctionDef_Node::_internal_dep() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.dep_;
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
FunctionDef_Node::_internal_mutable_dep() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.dep_;
}

// map<string, .opencv_tensorflow.AttrValue> attr = 5;
inline int FunctionDef_Node::_internal_attr_size() const {
  return _internal_attr().size();
}
inline int FunctionDef_Node::attr_size() const {
  return _internal_attr_size();
}
inline const ::google::protobuf::Map<::std::string, ::opencv_tensorflow::AttrValue>& FunctionDef_Node::_internal_attr() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.attr_.GetMap();
}
inline const ::google::protobuf::Map<::std::string, ::opencv_tensorflow::AttrValue>& FunctionDef_Node::attr() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_map:opencv_tensorflow.FunctionDef.Node.attr)
  return _internal_attr();
}
inline ::google::protobuf::Map<::std::string, ::opencv_tensorflow::AttrValue>* PROTOBUF_NONNULL FunctionDef_Node::_internal_mutable_attr() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.attr_.MutableMap();
}
inline ::google::protobuf::Map<::std::string, ::opencv_tensorflow::AttrValue>* PROTOBUF_NONNULL FunctionDef_Node::mutable_attr()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable_map:opencv_tensorflow.FunctionDef.Node.attr)
  return _internal_mutable_attr();
}

// -------------------------------------------------------------------

// FunctionDef

// .opencv_tensorflow.OpDef signature = 1;
inline bool FunctionDef::has_signature() const {
  bool value = (_impl_._has_bits_[0] & 0x00000001U) != 0;
  PROTOBUF_ASSUME(!value || _impl_.signature_ != nullptr);
  return value;
}
inline const ::opencv_tensorflow::OpDef& FunctionDef::_internal_signature() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::opencv_tensorflow::OpDef* p = _impl_.signature_;
  return p != nullptr ? *p : reinterpret_cast<const ::opencv_tensorflow::OpDef&>(::opencv_tensorflow::_OpDef_default_instance_);
}
inline const ::opencv_tensorflow::OpDef& FunctionDef::signature() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opencv_tensorflow.FunctionDef.signature)
  return _internal_signature();
}
inline void FunctionDef::unsafe_arena_set_allocated_signature(
    ::opencv_tensorflow::OpDef* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.signature_);
  }
  _impl_.signature_ = reinterpret_cast<::opencv_tensorflow::OpDef*>(value);
  if (value != nullptr) {
    _impl_._has_bits_[0] |= 0x00000001U;
  } else {
    _impl_._has_bits_[0] &= ~0x00000001U;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:opencv_tensorflow.FunctionDef.signature)
}
inline ::opencv_tensorflow::OpDef* PROTOBUF_NULLABLE FunctionDef::release_signature() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  _impl_._has_bits_[0] &= ~0x00000001U;
  ::opencv_tensorflow::OpDef* released = _impl_.signature_;
  _impl_.signature_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::opencv_tensorflow::OpDef* PROTOBUF_NULLABLE FunctionDef::unsafe_arena_release_signature() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:opencv_tensorflow.FunctionDef.signature)

  _impl_._has_bits_[0] &= ~0x00000001U;
  ::opencv_tensorflow::OpDef* temp = _impl_.signature_;
  _impl_.signature_ = nullptr;
  return temp;
}
inline ::opencv_tensorflow::OpDef* PROTOBUF_NONNULL FunctionDef::_internal_mutable_signature() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.signature_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::opencv_tensorflow::OpDef>(GetArena());
    _impl_.signature_ = reinterpret_cast<::opencv_tensorflow::OpDef*>(p);
  }
  return _impl_.signature_;
}
inline ::opencv_tensorflow::OpDef* PROTOBUF_NONNULL FunctionDef::mutable_signature()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  _impl_._has_bits_[0] |= 0x00000001U;
  ::opencv_tensorflow::OpDef* _msg = _internal_mutable_signature();
  // @@protoc_insertion_point(field_mutable:opencv_tensorflow.FunctionDef.signature)
  return _msg;
}
inline void FunctionDef::set_allocated_signature(::opencv_tensorflow::OpDef* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.signature_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    _impl_._has_bits_[0] |= 0x00000001U;
  } else {
    _impl_._has_bits_[0] &= ~0x00000001U;
  }

  _impl_.signature_ = reinterpret_cast<::opencv_tensorflow::OpDef*>(value);
  // @@protoc_insertion_point(field_set_allocated:opencv_tensorflow.FunctionDef.signature)
}

// repeated .opencv_tensorflow.FunctionDef.Node node = 2;
inline int FunctionDef::_internal_node_size() const {
  return _internal_node().size();
}
inline int FunctionDef::node_size() const {
  return _internal_node_size();
}
inline void FunctionDef::clear_node() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.node_.Clear();
}
inline ::opencv_tensorflow::FunctionDef_Node* PROTOBUF_NONNULL FunctionDef::mutable_node(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:opencv_tensorflow.FunctionDef.node)
  return _internal_mutable_node()->Mutable(index);
}
inline ::google::protobuf::RepeatedPtrField<::opencv_tensorflow::FunctionDef_Node>* PROTOBUF_NONNULL FunctionDef::mutable_node()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable_list:opencv_tensorflow.FunctionDef.node)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_node();
}
inline const ::opencv_tensorflow::FunctionDef_Node& FunctionDef::node(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opencv_tensorflow.FunctionDef.node)
  return _internal_node().Get(index);
}
inline ::opencv_tensorflow::FunctionDef_Node* PROTOBUF_NONNULL FunctionDef::add_node()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::opencv_tensorflow::FunctionDef_Node* _add = _internal_mutable_node()->Add();
  // @@protoc_insertion_point(field_add:opencv_tensorflow.FunctionDef.node)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::opencv_tensorflow::FunctionDef_Node>& FunctionDef::node() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opencv_tensorflow.FunctionDef.node)
  return _internal_node();
}
inline const ::google::protobuf::RepeatedPtrField<::opencv_tensorflow::FunctionDef_Node>&
FunctionDef::_internal_node() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.node_;
}
inline ::google::protobuf::RepeatedPtrField<::opencv_tensorflow::FunctionDef_Node>* PROTOBUF_NONNULL
FunctionDef::_internal_mutable_node() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.node_;
}

// -------------------------------------------------------------------

// GradientDef

// string function_name = 1;
inline void GradientDef::clear_function_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.function_name_.ClearToEmpty();
  _impl_._has_bits_[0] &= ~0x00000001U;
}
inline const ::std::string& GradientDef::function_name() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opencv_tensorflow.GradientDef.function_name)
  return _internal_function_name();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void GradientDef::set_function_name(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_._has_bits_[0] |= 0x00000001U;
  _impl_.function_name_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:opencv_tensorflow.GradientDef.function_name)
}
inline ::std::string* PROTOBUF_NONNULL GradientDef::mutable_function_name()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::std::string* _s = _internal_mutable_function_name();
  // @@protoc_insertion_point(field_mutable:opencv_tensorflow.GradientDef.function_name)
  return _s;
}
inline const ::std::string& GradientDef::_internal_function_name() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.function_name_.Get();
}
inline void GradientDef::_internal_set_function_name(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_._has_bits_[0] |= 0x00000001U;
  _impl_.function_name_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL GradientDef::_internal_mutable_function_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_._has_bits_[0] |= 0x00000001U;
  return _impl_.function_name_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE GradientDef::release_function_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:opencv_tensorflow.GradientDef.function_name)
  if ((_impl_._has_bits_[0] & 0x00000001U) == 0) {
    return nullptr;
  }
  _impl_._has_bits_[0] &= ~0x00000001U;
  auto* released = _impl_.function_name_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.function_name_.Set("", GetArena());
  }
  return released;
}
inline void GradientDef::set_allocated_function_name(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    _impl_._has_bits_[0] |= 0x00000001U;
  } else {
    _impl_._has_bits_[0] &= ~0x00000001U;
  }
  _impl_.function_name_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.function_name_.IsDefault()) {
    _impl_.function_name_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:opencv_tensorflow.GradientDef.function_name)
}

// string gradient_func = 2;
inline void GradientDef::clear_gradient_func() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.gradient_func_.ClearToEmpty();
  _impl_._has_bits_[0] &= ~0x00000002U;
}
inline const ::std::string& GradientDef::gradient_func() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opencv_tensorflow.GradientDef.gradient_func)
  return _internal_gradient_func();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void GradientDef::set_gradient_func(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_._has_bits_[0] |= 0x00000002U;
  _impl_.gradient_func_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:opencv_tensorflow.GradientDef.gradient_func)
}
inline ::std::string* PROTOBUF_NONNULL GradientDef::mutable_gradient_func()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::std::string* _s = _internal_mutable_gradient_func();
  // @@protoc_insertion_point(field_mutable:opencv_tensorflow.GradientDef.gradient_func)
  return _s;
}
inline const ::std::string& GradientDef::_internal_gradient_func() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.gradient_func_.Get();
}
inline void GradientDef::_internal_set_gradient_func(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_._has_bits_[0] |= 0x00000002U;
  _impl_.gradient_func_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL GradientDef::_internal_mutable_gradient_func() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_._has_bits_[0] |= 0x00000002U;
  return _impl_.gradient_func_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE GradientDef::release_gradient_func() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:opencv_tensorflow.GradientDef.gradient_func)
  if ((_impl_._has_bits_[0] & 0x00000002U) == 0) {
    return nullptr;
  }
  _impl_._has_bits_[0] &= ~0x00000002U;
  auto* released = _impl_.gradient_func_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.gradient_func_.Set("", GetArena());
  }
  return released;
}
inline void GradientDef::set_allocated_gradient_func(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    _impl_._has_bits_[0] |= 0x00000002U;
  } else {
    _impl_._has_bits_[0] &= ~0x00000002U;
  }
  _impl_.gradient_func_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.gradient_func_.IsDefault()) {
    _impl_.gradient_func_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:opencv_tensorflow.GradientDef.gradient_func)
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace opencv_tensorflow


// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"

#endif  // function_2eproto_2epb_2eh
