/****************************************************************************
** Meta object code from reading C++ file 'cvvkeypoint.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../opencv_contrib/modules/cvv/src/qtutil/matchview/cvvkeypoint.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'cvvkeypoint.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3cvv6qtutil11CVVKeyPointE_t {};
} // unnamed namespace

template <> constexpr inline auto cvv::qtutil::CVVKeyPoint::qt_create_metaobjectdata<qt_meta_tag_ZN3cvv6qtutil11CVVKeyPointE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "cvv::qtutil::CVVKeyPoint",
        "updatePoint",
        "",
        "visible",
        "updateSettings",
        "KeyPointSettings&",
        "settings",
        "setPen",
        "QPen",
        "pen",
        "setBrush",
        "QBrush",
        "brush",
        "setShow",
        "b",
        "updateImageSet",
        "QRectF",
        "zoom",
        "setZoomableImage",
        "ZoomableImage*",
        "image"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'updatePoint'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'updateSettings'
        QtMocHelpers::SlotData<void(KeyPointSettings &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'setPen'
        QtMocHelpers::SlotData<void(const QPen &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'setBrush'
        QtMocHelpers::SlotData<void(const QBrush &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'setShow'
        QtMocHelpers::SlotData<void(bool)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
        // Slot 'updateImageSet'
        QtMocHelpers::SlotData<void(const QRectF &, const qreal &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 2 }, { QMetaType::QReal, 17 },
        }}),
        // Slot 'setZoomableImage'
        QtMocHelpers::SlotData<void(ZoomableImage *)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CVVKeyPoint, qt_meta_tag_ZN3cvv6qtutil11CVVKeyPointE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject cvv::qtutil::CVVKeyPoint::staticMetaObject = { {
    QMetaObject::SuperData::link<QGraphicsObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3cvv6qtutil11CVVKeyPointE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3cvv6qtutil11CVVKeyPointE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3cvv6qtutil11CVVKeyPointE_t>.metaTypes,
    nullptr
} };

void cvv::qtutil::CVVKeyPoint::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CVVKeyPoint *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->updatePoint((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->updateSettings((*reinterpret_cast<std::add_pointer_t<KeyPointSettings&>>(_a[1]))); break;
        case 2: _t->setPen((*reinterpret_cast<std::add_pointer_t<QPen>>(_a[1]))); break;
        case 3: _t->setBrush((*reinterpret_cast<std::add_pointer_t<QBrush>>(_a[1]))); break;
        case 4: _t->setShow((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->updateImageSet((*reinterpret_cast<std::add_pointer_t<QRectF>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2]))); break;
        case 6: _t->setZoomableImage((*reinterpret_cast<std::add_pointer_t<ZoomableImage*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< ZoomableImage* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CVVKeyPoint::*)(bool )>(_a, &CVVKeyPoint::updatePoint, 0))
            return;
    }
}

const QMetaObject *cvv::qtutil::CVVKeyPoint::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *cvv::qtutil::CVVKeyPoint::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3cvv6qtutil11CVVKeyPointE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "cv::KeyPoint"))
        return static_cast< cv::KeyPoint*>(this);
    return QGraphicsObject::qt_metacast(_clname);
}

int cvv::qtutil::CVVKeyPoint::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QGraphicsObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void cvv::qtutil::CVVKeyPoint::updatePoint(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
