// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: tensor.proto
// Protobuf C++ Version: 6.32.1

#ifndef tensor_2eproto_2epb_2eh
#define tensor_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 6032001
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/message.h"
#include "google/protobuf/message_lite.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/unknown_field_set.h"
#include "tensor_shape.pb.h"
#include "types.pb.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_tensor_2eproto

namespace google {
namespace protobuf {
namespace internal {
template <typename T>
::absl::string_view GetAnyMessageName();
}  // namespace internal
}  // namespace protobuf
}  // namespace google

// Internal implementation detail -- do not use these members.
struct TableStruct_tensor_2eproto {
  static const ::uint32_t offsets[];
};
extern "C" {
extern const ::google::protobuf::internal::DescriptorTable descriptor_table_tensor_2eproto;
}  // extern "C"
namespace opencv_tensorflow {
class TensorProto;
struct TensorProtoDefaultTypeInternal;
extern TensorProtoDefaultTypeInternal _TensorProto_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull TensorProto_class_data_;
}  // namespace opencv_tensorflow
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google

namespace opencv_tensorflow {

// ===================================================================


// -------------------------------------------------------------------

class TensorProto final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:opencv_tensorflow.TensorProto) */ {
 public:
  inline TensorProto() : TensorProto(nullptr) {}
  ~TensorProto() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(TensorProto* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(TensorProto));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR TensorProto(::google::protobuf::internal::ConstantInitialized);

  inline TensorProto(const TensorProto& from) : TensorProto(nullptr, from) {}
  inline TensorProto(TensorProto&& from) noexcept
      : TensorProto(nullptr, ::std::move(from)) {}
  inline TensorProto& operator=(const TensorProto& from) {
    CopyFrom(from);
    return *this;
  }
  inline TensorProto& operator=(TensorProto&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const TensorProto& default_instance() {
    return *reinterpret_cast<const TensorProto*>(
        &_TensorProto_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(TensorProto& a, TensorProto& b) { a.Swap(&b); }
  inline void Swap(TensorProto* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(TensorProto* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  TensorProto* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<TensorProto>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const TensorProto& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const TensorProto& from) { TensorProto::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(TensorProto* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "opencv_tensorflow.TensorProto"; }

 protected:
  explicit TensorProto(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  TensorProto(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const TensorProto& from);
  TensorProto(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, TensorProto&& from) noexcept
      : TensorProto(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kFloatValFieldNumber = 5,
    kDoubleValFieldNumber = 6,
    kIntValFieldNumber = 7,
    kStringValFieldNumber = 8,
    kScomplexValFieldNumber = 9,
    kInt64ValFieldNumber = 10,
    kBoolValFieldNumber = 11,
    kDcomplexValFieldNumber = 12,
    kHalfValFieldNumber = 13,
    kTensorContentFieldNumber = 4,
    kTensorShapeFieldNumber = 2,
    kDtypeFieldNumber = 1,
    kVersionNumberFieldNumber = 3,
  };
  // repeated float float_val = 5 [packed = true];
  int float_val_size() const;
  private:
  int _internal_float_val_size() const;

  public:
  void clear_float_val() ;
  float float_val(int index) const;
  void set_float_val(int index, float value);
  void add_float_val(float value);
  const ::google::protobuf::RepeatedField<float>& float_val() const;
  ::google::protobuf::RepeatedField<float>* PROTOBUF_NONNULL mutable_float_val();

  private:
  const ::google::protobuf::RepeatedField<float>& _internal_float_val() const;
  ::google::protobuf::RepeatedField<float>* PROTOBUF_NONNULL _internal_mutable_float_val();

  public:
  // repeated double double_val = 6 [packed = true];
  int double_val_size() const;
  private:
  int _internal_double_val_size() const;

  public:
  void clear_double_val() ;
  double double_val(int index) const;
  void set_double_val(int index, double value);
  void add_double_val(double value);
  const ::google::protobuf::RepeatedField<double>& double_val() const;
  ::google::protobuf::RepeatedField<double>* PROTOBUF_NONNULL mutable_double_val();

  private:
  const ::google::protobuf::RepeatedField<double>& _internal_double_val() const;
  ::google::protobuf::RepeatedField<double>* PROTOBUF_NONNULL _internal_mutable_double_val();

  public:
  // repeated int32 int_val = 7 [packed = true];
  int int_val_size() const;
  private:
  int _internal_int_val_size() const;

  public:
  void clear_int_val() ;
  ::int32_t int_val(int index) const;
  void set_int_val(int index, ::int32_t value);
  void add_int_val(::int32_t value);
  const ::google::protobuf::RepeatedField<::int32_t>& int_val() const;
  ::google::protobuf::RepeatedField<::int32_t>* PROTOBUF_NONNULL mutable_int_val();

  private:
  const ::google::protobuf::RepeatedField<::int32_t>& _internal_int_val() const;
  ::google::protobuf::RepeatedField<::int32_t>* PROTOBUF_NONNULL _internal_mutable_int_val();

  public:
  // repeated bytes string_val = 8;
  int string_val_size() const;
  private:
  int _internal_string_val_size() const;

  public:
  void clear_string_val() ;
  const ::std::string& string_val(int index) const;
  ::std::string* PROTOBUF_NONNULL mutable_string_val(int index);
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_string_val(int index, Arg_&& value, Args_... args);
  ::std::string* PROTOBUF_NONNULL add_string_val();
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void add_string_val(Arg_&& value, Args_... args);
  const ::google::protobuf::RepeatedPtrField<::std::string>& string_val() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL mutable_string_val();

  private:
  const ::google::protobuf::RepeatedPtrField<::std::string>& _internal_string_val() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL _internal_mutable_string_val();

  public:
  // repeated float scomplex_val = 9 [packed = true];
  int scomplex_val_size() const;
  private:
  int _internal_scomplex_val_size() const;

  public:
  void clear_scomplex_val() ;
  float scomplex_val(int index) const;
  void set_scomplex_val(int index, float value);
  void add_scomplex_val(float value);
  const ::google::protobuf::RepeatedField<float>& scomplex_val() const;
  ::google::protobuf::RepeatedField<float>* PROTOBUF_NONNULL mutable_scomplex_val();

  private:
  const ::google::protobuf::RepeatedField<float>& _internal_scomplex_val() const;
  ::google::protobuf::RepeatedField<float>* PROTOBUF_NONNULL _internal_mutable_scomplex_val();

  public:
  // repeated int64 int64_val = 10 [packed = true];
  int int64_val_size() const;
  private:
  int _internal_int64_val_size() const;

  public:
  void clear_int64_val() ;
  ::int64_t int64_val(int index) const;
  void set_int64_val(int index, ::int64_t value);
  void add_int64_val(::int64_t value);
  const ::google::protobuf::RepeatedField<::int64_t>& int64_val() const;
  ::google::protobuf::RepeatedField<::int64_t>* PROTOBUF_NONNULL mutable_int64_val();

  private:
  const ::google::protobuf::RepeatedField<::int64_t>& _internal_int64_val() const;
  ::google::protobuf::RepeatedField<::int64_t>* PROTOBUF_NONNULL _internal_mutable_int64_val();

  public:
  // repeated bool bool_val = 11 [packed = true];
  int bool_val_size() const;
  private:
  int _internal_bool_val_size() const;

  public:
  void clear_bool_val() ;
  bool bool_val(int index) const;
  void set_bool_val(int index, bool value);
  void add_bool_val(bool value);
  const ::google::protobuf::RepeatedField<bool>& bool_val() const;
  ::google::protobuf::RepeatedField<bool>* PROTOBUF_NONNULL mutable_bool_val();

  private:
  const ::google::protobuf::RepeatedField<bool>& _internal_bool_val() const;
  ::google::protobuf::RepeatedField<bool>* PROTOBUF_NONNULL _internal_mutable_bool_val();

  public:
  // repeated double dcomplex_val = 12 [packed = true];
  int dcomplex_val_size() const;
  private:
  int _internal_dcomplex_val_size() const;

  public:
  void clear_dcomplex_val() ;
  double dcomplex_val(int index) const;
  void set_dcomplex_val(int index, double value);
  void add_dcomplex_val(double value);
  const ::google::protobuf::RepeatedField<double>& dcomplex_val() const;
  ::google::protobuf::RepeatedField<double>* PROTOBUF_NONNULL mutable_dcomplex_val();

  private:
  const ::google::protobuf::RepeatedField<double>& _internal_dcomplex_val() const;
  ::google::protobuf::RepeatedField<double>* PROTOBUF_NONNULL _internal_mutable_dcomplex_val();

  public:
  // repeated int32 half_val = 13 [packed = true];
  int half_val_size() const;
  private:
  int _internal_half_val_size() const;

  public:
  void clear_half_val() ;
  ::int32_t half_val(int index) const;
  void set_half_val(int index, ::int32_t value);
  void add_half_val(::int32_t value);
  const ::google::protobuf::RepeatedField<::int32_t>& half_val() const;
  ::google::protobuf::RepeatedField<::int32_t>* PROTOBUF_NONNULL mutable_half_val();

  private:
  const ::google::protobuf::RepeatedField<::int32_t>& _internal_half_val() const;
  ::google::protobuf::RepeatedField<::int32_t>* PROTOBUF_NONNULL _internal_mutable_half_val();

  public:
  // bytes tensor_content = 4;
  void clear_tensor_content() ;
  const ::std::string& tensor_content() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_tensor_content(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_tensor_content();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_tensor_content();
  void set_allocated_tensor_content(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_tensor_content() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_tensor_content(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_tensor_content();

  public:
  // .opencv_tensorflow.TensorShapeProto tensor_shape = 2;
  bool has_tensor_shape() const;
  void clear_tensor_shape() ;
  const ::opencv_tensorflow::TensorShapeProto& tensor_shape() const;
  [[nodiscard]] ::opencv_tensorflow::TensorShapeProto* PROTOBUF_NULLABLE release_tensor_shape();
  ::opencv_tensorflow::TensorShapeProto* PROTOBUF_NONNULL mutable_tensor_shape();
  void set_allocated_tensor_shape(::opencv_tensorflow::TensorShapeProto* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_tensor_shape(::opencv_tensorflow::TensorShapeProto* PROTOBUF_NULLABLE value);
  ::opencv_tensorflow::TensorShapeProto* PROTOBUF_NULLABLE unsafe_arena_release_tensor_shape();

  private:
  const ::opencv_tensorflow::TensorShapeProto& _internal_tensor_shape() const;
  ::opencv_tensorflow::TensorShapeProto* PROTOBUF_NONNULL _internal_mutable_tensor_shape();

  public:
  // .opencv_tensorflow.DataType dtype = 1;
  void clear_dtype() ;
  ::opencv_tensorflow::DataType dtype() const;
  void set_dtype(::opencv_tensorflow::DataType value);

  private:
  ::opencv_tensorflow::DataType _internal_dtype() const;
  void _internal_set_dtype(::opencv_tensorflow::DataType value);

  public:
  // int32 version_number = 3;
  void clear_version_number() ;
  ::int32_t version_number() const;
  void set_version_number(::int32_t value);

  private:
  ::int32_t _internal_version_number() const;
  void _internal_set_version_number(::int32_t value);

  public:
  // @@protoc_insertion_point(class_scope:opencv_tensorflow.TensorProto)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<4, 13,
                                   1, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const TensorProto& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedField<float> float_val_;
    ::google::protobuf::RepeatedField<double> double_val_;
    ::google::protobuf::RepeatedField<::int32_t> int_val_;
    ::google::protobuf::internal::CachedSize _int_val_cached_byte_size_;
    ::google::protobuf::RepeatedPtrField<::std::string> string_val_;
    ::google::protobuf::RepeatedField<float> scomplex_val_;
    ::google::protobuf::RepeatedField<::int64_t> int64_val_;
    ::google::protobuf::internal::CachedSize _int64_val_cached_byte_size_;
    ::google::protobuf::RepeatedField<bool> bool_val_;
    ::google::protobuf::RepeatedField<double> dcomplex_val_;
    ::google::protobuf::RepeatedField<::int32_t> half_val_;
    ::google::protobuf::internal::CachedSize _half_val_cached_byte_size_;
    ::google::protobuf::internal::ArenaStringPtr tensor_content_;
    ::opencv_tensorflow::TensorShapeProto* PROTOBUF_NULLABLE tensor_shape_;
    int dtype_;
    ::int32_t version_number_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_tensor_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull TensorProto_class_data_;

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// TensorProto

// .opencv_tensorflow.DataType dtype = 1;
inline void TensorProto::clear_dtype() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.dtype_ = 0;
  _impl_._has_bits_[0] &= ~0x00000004U;
}
inline ::opencv_tensorflow::DataType TensorProto::dtype() const {
  // @@protoc_insertion_point(field_get:opencv_tensorflow.TensorProto.dtype)
  return _internal_dtype();
}
inline void TensorProto::set_dtype(::opencv_tensorflow::DataType value) {
  _internal_set_dtype(value);
  _impl_._has_bits_[0] |= 0x00000004U;
  // @@protoc_insertion_point(field_set:opencv_tensorflow.TensorProto.dtype)
}
inline ::opencv_tensorflow::DataType TensorProto::_internal_dtype() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::opencv_tensorflow::DataType>(_impl_.dtype_);
}
inline void TensorProto::_internal_set_dtype(::opencv_tensorflow::DataType value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.dtype_ = value;
}

// .opencv_tensorflow.TensorShapeProto tensor_shape = 2;
inline bool TensorProto::has_tensor_shape() const {
  bool value = (_impl_._has_bits_[0] & 0x00000002U) != 0;
  PROTOBUF_ASSUME(!value || _impl_.tensor_shape_ != nullptr);
  return value;
}
inline const ::opencv_tensorflow::TensorShapeProto& TensorProto::_internal_tensor_shape() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::opencv_tensorflow::TensorShapeProto* p = _impl_.tensor_shape_;
  return p != nullptr ? *p : reinterpret_cast<const ::opencv_tensorflow::TensorShapeProto&>(::opencv_tensorflow::_TensorShapeProto_default_instance_);
}
inline const ::opencv_tensorflow::TensorShapeProto& TensorProto::tensor_shape() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opencv_tensorflow.TensorProto.tensor_shape)
  return _internal_tensor_shape();
}
inline void TensorProto::unsafe_arena_set_allocated_tensor_shape(
    ::opencv_tensorflow::TensorShapeProto* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.tensor_shape_);
  }
  _impl_.tensor_shape_ = reinterpret_cast<::opencv_tensorflow::TensorShapeProto*>(value);
  if (value != nullptr) {
    _impl_._has_bits_[0] |= 0x00000002U;
  } else {
    _impl_._has_bits_[0] &= ~0x00000002U;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:opencv_tensorflow.TensorProto.tensor_shape)
}
inline ::opencv_tensorflow::TensorShapeProto* PROTOBUF_NULLABLE TensorProto::release_tensor_shape() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  _impl_._has_bits_[0] &= ~0x00000002U;
  ::opencv_tensorflow::TensorShapeProto* released = _impl_.tensor_shape_;
  _impl_.tensor_shape_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::opencv_tensorflow::TensorShapeProto* PROTOBUF_NULLABLE TensorProto::unsafe_arena_release_tensor_shape() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:opencv_tensorflow.TensorProto.tensor_shape)

  _impl_._has_bits_[0] &= ~0x00000002U;
  ::opencv_tensorflow::TensorShapeProto* temp = _impl_.tensor_shape_;
  _impl_.tensor_shape_ = nullptr;
  return temp;
}
inline ::opencv_tensorflow::TensorShapeProto* PROTOBUF_NONNULL TensorProto::_internal_mutable_tensor_shape() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.tensor_shape_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::opencv_tensorflow::TensorShapeProto>(GetArena());
    _impl_.tensor_shape_ = reinterpret_cast<::opencv_tensorflow::TensorShapeProto*>(p);
  }
  return _impl_.tensor_shape_;
}
inline ::opencv_tensorflow::TensorShapeProto* PROTOBUF_NONNULL TensorProto::mutable_tensor_shape()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  _impl_._has_bits_[0] |= 0x00000002U;
  ::opencv_tensorflow::TensorShapeProto* _msg = _internal_mutable_tensor_shape();
  // @@protoc_insertion_point(field_mutable:opencv_tensorflow.TensorProto.tensor_shape)
  return _msg;
}
inline void TensorProto::set_allocated_tensor_shape(::opencv_tensorflow::TensorShapeProto* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.tensor_shape_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    _impl_._has_bits_[0] |= 0x00000002U;
  } else {
    _impl_._has_bits_[0] &= ~0x00000002U;
  }

  _impl_.tensor_shape_ = reinterpret_cast<::opencv_tensorflow::TensorShapeProto*>(value);
  // @@protoc_insertion_point(field_set_allocated:opencv_tensorflow.TensorProto.tensor_shape)
}

// int32 version_number = 3;
inline void TensorProto::clear_version_number() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.version_number_ = 0;
  _impl_._has_bits_[0] &= ~0x00000008U;
}
inline ::int32_t TensorProto::version_number() const {
  // @@protoc_insertion_point(field_get:opencv_tensorflow.TensorProto.version_number)
  return _internal_version_number();
}
inline void TensorProto::set_version_number(::int32_t value) {
  _internal_set_version_number(value);
  _impl_._has_bits_[0] |= 0x00000008U;
  // @@protoc_insertion_point(field_set:opencv_tensorflow.TensorProto.version_number)
}
inline ::int32_t TensorProto::_internal_version_number() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.version_number_;
}
inline void TensorProto::_internal_set_version_number(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.version_number_ = value;
}

// bytes tensor_content = 4;
inline void TensorProto::clear_tensor_content() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.tensor_content_.ClearToEmpty();
  _impl_._has_bits_[0] &= ~0x00000001U;
}
inline const ::std::string& TensorProto::tensor_content() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opencv_tensorflow.TensorProto.tensor_content)
  return _internal_tensor_content();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void TensorProto::set_tensor_content(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_._has_bits_[0] |= 0x00000001U;
  _impl_.tensor_content_.SetBytes(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:opencv_tensorflow.TensorProto.tensor_content)
}
inline ::std::string* PROTOBUF_NONNULL TensorProto::mutable_tensor_content()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::std::string* _s = _internal_mutable_tensor_content();
  // @@protoc_insertion_point(field_mutable:opencv_tensorflow.TensorProto.tensor_content)
  return _s;
}
inline const ::std::string& TensorProto::_internal_tensor_content() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.tensor_content_.Get();
}
inline void TensorProto::_internal_set_tensor_content(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_._has_bits_[0] |= 0x00000001U;
  _impl_.tensor_content_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL TensorProto::_internal_mutable_tensor_content() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_._has_bits_[0] |= 0x00000001U;
  return _impl_.tensor_content_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE TensorProto::release_tensor_content() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:opencv_tensorflow.TensorProto.tensor_content)
  if ((_impl_._has_bits_[0] & 0x00000001U) == 0) {
    return nullptr;
  }
  _impl_._has_bits_[0] &= ~0x00000001U;
  auto* released = _impl_.tensor_content_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.tensor_content_.Set("", GetArena());
  }
  return released;
}
inline void TensorProto::set_allocated_tensor_content(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    _impl_._has_bits_[0] |= 0x00000001U;
  } else {
    _impl_._has_bits_[0] &= ~0x00000001U;
  }
  _impl_.tensor_content_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.tensor_content_.IsDefault()) {
    _impl_.tensor_content_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:opencv_tensorflow.TensorProto.tensor_content)
}

// repeated int32 half_val = 13 [packed = true];
inline int TensorProto::_internal_half_val_size() const {
  return _internal_half_val().size();
}
inline int TensorProto::half_val_size() const {
  return _internal_half_val_size();
}
inline void TensorProto::clear_half_val() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.half_val_.Clear();
}
inline ::int32_t TensorProto::half_val(int index) const {
  // @@protoc_insertion_point(field_get:opencv_tensorflow.TensorProto.half_val)
  return _internal_half_val().Get(index);
}
inline void TensorProto::set_half_val(int index, ::int32_t value) {
  _internal_mutable_half_val()->Set(index, value);
  // @@protoc_insertion_point(field_set:opencv_tensorflow.TensorProto.half_val)
}
inline void TensorProto::add_half_val(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _internal_mutable_half_val()->Add(value);
  // @@protoc_insertion_point(field_add:opencv_tensorflow.TensorProto.half_val)
}
inline const ::google::protobuf::RepeatedField<::int32_t>& TensorProto::half_val() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opencv_tensorflow.TensorProto.half_val)
  return _internal_half_val();
}
inline ::google::protobuf::RepeatedField<::int32_t>* PROTOBUF_NONNULL TensorProto::mutable_half_val()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable_list:opencv_tensorflow.TensorProto.half_val)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_half_val();
}
inline const ::google::protobuf::RepeatedField<::int32_t>&
TensorProto::_internal_half_val() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.half_val_;
}
inline ::google::protobuf::RepeatedField<::int32_t>* PROTOBUF_NONNULL
TensorProto::_internal_mutable_half_val() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.half_val_;
}

// repeated float float_val = 5 [packed = true];
inline int TensorProto::_internal_float_val_size() const {
  return _internal_float_val().size();
}
inline int TensorProto::float_val_size() const {
  return _internal_float_val_size();
}
inline void TensorProto::clear_float_val() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.float_val_.Clear();
}
inline float TensorProto::float_val(int index) const {
  // @@protoc_insertion_point(field_get:opencv_tensorflow.TensorProto.float_val)
  return _internal_float_val().Get(index);
}
inline void TensorProto::set_float_val(int index, float value) {
  _internal_mutable_float_val()->Set(index, value);
  // @@protoc_insertion_point(field_set:opencv_tensorflow.TensorProto.float_val)
}
inline void TensorProto::add_float_val(float value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _internal_mutable_float_val()->Add(value);
  // @@protoc_insertion_point(field_add:opencv_tensorflow.TensorProto.float_val)
}
inline const ::google::protobuf::RepeatedField<float>& TensorProto::float_val() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opencv_tensorflow.TensorProto.float_val)
  return _internal_float_val();
}
inline ::google::protobuf::RepeatedField<float>* PROTOBUF_NONNULL TensorProto::mutable_float_val()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable_list:opencv_tensorflow.TensorProto.float_val)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_float_val();
}
inline const ::google::protobuf::RepeatedField<float>&
TensorProto::_internal_float_val() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.float_val_;
}
inline ::google::protobuf::RepeatedField<float>* PROTOBUF_NONNULL
TensorProto::_internal_mutable_float_val() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.float_val_;
}

// repeated double double_val = 6 [packed = true];
inline int TensorProto::_internal_double_val_size() const {
  return _internal_double_val().size();
}
inline int TensorProto::double_val_size() const {
  return _internal_double_val_size();
}
inline void TensorProto::clear_double_val() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.double_val_.Clear();
}
inline double TensorProto::double_val(int index) const {
  // @@protoc_insertion_point(field_get:opencv_tensorflow.TensorProto.double_val)
  return _internal_double_val().Get(index);
}
inline void TensorProto::set_double_val(int index, double value) {
  _internal_mutable_double_val()->Set(index, value);
  // @@protoc_insertion_point(field_set:opencv_tensorflow.TensorProto.double_val)
}
inline void TensorProto::add_double_val(double value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _internal_mutable_double_val()->Add(value);
  // @@protoc_insertion_point(field_add:opencv_tensorflow.TensorProto.double_val)
}
inline const ::google::protobuf::RepeatedField<double>& TensorProto::double_val() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opencv_tensorflow.TensorProto.double_val)
  return _internal_double_val();
}
inline ::google::protobuf::RepeatedField<double>* PROTOBUF_NONNULL TensorProto::mutable_double_val()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable_list:opencv_tensorflow.TensorProto.double_val)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_double_val();
}
inline const ::google::protobuf::RepeatedField<double>&
TensorProto::_internal_double_val() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.double_val_;
}
inline ::google::protobuf::RepeatedField<double>* PROTOBUF_NONNULL
TensorProto::_internal_mutable_double_val() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.double_val_;
}

// repeated int32 int_val = 7 [packed = true];
inline int TensorProto::_internal_int_val_size() const {
  return _internal_int_val().size();
}
inline int TensorProto::int_val_size() const {
  return _internal_int_val_size();
}
inline void TensorProto::clear_int_val() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.int_val_.Clear();
}
inline ::int32_t TensorProto::int_val(int index) const {
  // @@protoc_insertion_point(field_get:opencv_tensorflow.TensorProto.int_val)
  return _internal_int_val().Get(index);
}
inline void TensorProto::set_int_val(int index, ::int32_t value) {
  _internal_mutable_int_val()->Set(index, value);
  // @@protoc_insertion_point(field_set:opencv_tensorflow.TensorProto.int_val)
}
inline void TensorProto::add_int_val(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _internal_mutable_int_val()->Add(value);
  // @@protoc_insertion_point(field_add:opencv_tensorflow.TensorProto.int_val)
}
inline const ::google::protobuf::RepeatedField<::int32_t>& TensorProto::int_val() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opencv_tensorflow.TensorProto.int_val)
  return _internal_int_val();
}
inline ::google::protobuf::RepeatedField<::int32_t>* PROTOBUF_NONNULL TensorProto::mutable_int_val()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable_list:opencv_tensorflow.TensorProto.int_val)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_int_val();
}
inline const ::google::protobuf::RepeatedField<::int32_t>&
TensorProto::_internal_int_val() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.int_val_;
}
inline ::google::protobuf::RepeatedField<::int32_t>* PROTOBUF_NONNULL
TensorProto::_internal_mutable_int_val() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.int_val_;
}

// repeated bytes string_val = 8;
inline int TensorProto::_internal_string_val_size() const {
  return _internal_string_val().size();
}
inline int TensorProto::string_val_size() const {
  return _internal_string_val_size();
}
inline void TensorProto::clear_string_val() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.string_val_.Clear();
}
inline ::std::string* PROTOBUF_NONNULL TensorProto::add_string_val()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::std::string* _s = _internal_mutable_string_val()->Add();
  // @@protoc_insertion_point(field_add_mutable:opencv_tensorflow.TensorProto.string_val)
  return _s;
}
inline const ::std::string& TensorProto::string_val(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opencv_tensorflow.TensorProto.string_val)
  return _internal_string_val().Get(index);
}
inline ::std::string* PROTOBUF_NONNULL TensorProto::mutable_string_val(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:opencv_tensorflow.TensorProto.string_val)
  return _internal_mutable_string_val()->Mutable(index);
}
template <typename Arg_, typename... Args_>
inline void TensorProto::set_string_val(int index, Arg_&& value, Args_... args) {
  ::google::protobuf::internal::AssignToString(*_internal_mutable_string_val()->Mutable(index), ::std::forward<Arg_>(value),
                        args... , ::google::protobuf::internal::BytesTag{});
  // @@protoc_insertion_point(field_set:opencv_tensorflow.TensorProto.string_val)
}
template <typename Arg_, typename... Args_>
inline void TensorProto::add_string_val(Arg_&& value, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::google::protobuf::internal::AddToRepeatedPtrField(*_internal_mutable_string_val(),
                               ::std::forward<Arg_>(value),
                               args... , ::google::protobuf::internal::BytesTag{});
  // @@protoc_insertion_point(field_add:opencv_tensorflow.TensorProto.string_val)
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>& TensorProto::string_val()
    const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opencv_tensorflow.TensorProto.string_val)
  return _internal_string_val();
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
TensorProto::mutable_string_val() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable_list:opencv_tensorflow.TensorProto.string_val)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_string_val();
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>&
TensorProto::_internal_string_val() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.string_val_;
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
TensorProto::_internal_mutable_string_val() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.string_val_;
}

// repeated float scomplex_val = 9 [packed = true];
inline int TensorProto::_internal_scomplex_val_size() const {
  return _internal_scomplex_val().size();
}
inline int TensorProto::scomplex_val_size() const {
  return _internal_scomplex_val_size();
}
inline void TensorProto::clear_scomplex_val() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.scomplex_val_.Clear();
}
inline float TensorProto::scomplex_val(int index) const {
  // @@protoc_insertion_point(field_get:opencv_tensorflow.TensorProto.scomplex_val)
  return _internal_scomplex_val().Get(index);
}
inline void TensorProto::set_scomplex_val(int index, float value) {
  _internal_mutable_scomplex_val()->Set(index, value);
  // @@protoc_insertion_point(field_set:opencv_tensorflow.TensorProto.scomplex_val)
}
inline void TensorProto::add_scomplex_val(float value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _internal_mutable_scomplex_val()->Add(value);
  // @@protoc_insertion_point(field_add:opencv_tensorflow.TensorProto.scomplex_val)
}
inline const ::google::protobuf::RepeatedField<float>& TensorProto::scomplex_val() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opencv_tensorflow.TensorProto.scomplex_val)
  return _internal_scomplex_val();
}
inline ::google::protobuf::RepeatedField<float>* PROTOBUF_NONNULL TensorProto::mutable_scomplex_val()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable_list:opencv_tensorflow.TensorProto.scomplex_val)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_scomplex_val();
}
inline const ::google::protobuf::RepeatedField<float>&
TensorProto::_internal_scomplex_val() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.scomplex_val_;
}
inline ::google::protobuf::RepeatedField<float>* PROTOBUF_NONNULL
TensorProto::_internal_mutable_scomplex_val() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.scomplex_val_;
}

// repeated int64 int64_val = 10 [packed = true];
inline int TensorProto::_internal_int64_val_size() const {
  return _internal_int64_val().size();
}
inline int TensorProto::int64_val_size() const {
  return _internal_int64_val_size();
}
inline void TensorProto::clear_int64_val() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.int64_val_.Clear();
}
inline ::int64_t TensorProto::int64_val(int index) const {
  // @@protoc_insertion_point(field_get:opencv_tensorflow.TensorProto.int64_val)
  return _internal_int64_val().Get(index);
}
inline void TensorProto::set_int64_val(int index, ::int64_t value) {
  _internal_mutable_int64_val()->Set(index, value);
  // @@protoc_insertion_point(field_set:opencv_tensorflow.TensorProto.int64_val)
}
inline void TensorProto::add_int64_val(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _internal_mutable_int64_val()->Add(value);
  // @@protoc_insertion_point(field_add:opencv_tensorflow.TensorProto.int64_val)
}
inline const ::google::protobuf::RepeatedField<::int64_t>& TensorProto::int64_val() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opencv_tensorflow.TensorProto.int64_val)
  return _internal_int64_val();
}
inline ::google::protobuf::RepeatedField<::int64_t>* PROTOBUF_NONNULL TensorProto::mutable_int64_val()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable_list:opencv_tensorflow.TensorProto.int64_val)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_int64_val();
}
inline const ::google::protobuf::RepeatedField<::int64_t>&
TensorProto::_internal_int64_val() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.int64_val_;
}
inline ::google::protobuf::RepeatedField<::int64_t>* PROTOBUF_NONNULL
TensorProto::_internal_mutable_int64_val() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.int64_val_;
}

// repeated bool bool_val = 11 [packed = true];
inline int TensorProto::_internal_bool_val_size() const {
  return _internal_bool_val().size();
}
inline int TensorProto::bool_val_size() const {
  return _internal_bool_val_size();
}
inline void TensorProto::clear_bool_val() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.bool_val_.Clear();
}
inline bool TensorProto::bool_val(int index) const {
  // @@protoc_insertion_point(field_get:opencv_tensorflow.TensorProto.bool_val)
  return _internal_bool_val().Get(index);
}
inline void TensorProto::set_bool_val(int index, bool value) {
  _internal_mutable_bool_val()->Set(index, value);
  // @@protoc_insertion_point(field_set:opencv_tensorflow.TensorProto.bool_val)
}
inline void TensorProto::add_bool_val(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _internal_mutable_bool_val()->Add(value);
  // @@protoc_insertion_point(field_add:opencv_tensorflow.TensorProto.bool_val)
}
inline const ::google::protobuf::RepeatedField<bool>& TensorProto::bool_val() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opencv_tensorflow.TensorProto.bool_val)
  return _internal_bool_val();
}
inline ::google::protobuf::RepeatedField<bool>* PROTOBUF_NONNULL TensorProto::mutable_bool_val()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable_list:opencv_tensorflow.TensorProto.bool_val)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_bool_val();
}
inline const ::google::protobuf::RepeatedField<bool>&
TensorProto::_internal_bool_val() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.bool_val_;
}
inline ::google::protobuf::RepeatedField<bool>* PROTOBUF_NONNULL
TensorProto::_internal_mutable_bool_val() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.bool_val_;
}

// repeated double dcomplex_val = 12 [packed = true];
inline int TensorProto::_internal_dcomplex_val_size() const {
  return _internal_dcomplex_val().size();
}
inline int TensorProto::dcomplex_val_size() const {
  return _internal_dcomplex_val_size();
}
inline void TensorProto::clear_dcomplex_val() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.dcomplex_val_.Clear();
}
inline double TensorProto::dcomplex_val(int index) const {
  // @@protoc_insertion_point(field_get:opencv_tensorflow.TensorProto.dcomplex_val)
  return _internal_dcomplex_val().Get(index);
}
inline void TensorProto::set_dcomplex_val(int index, double value) {
  _internal_mutable_dcomplex_val()->Set(index, value);
  // @@protoc_insertion_point(field_set:opencv_tensorflow.TensorProto.dcomplex_val)
}
inline void TensorProto::add_dcomplex_val(double value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _internal_mutable_dcomplex_val()->Add(value);
  // @@protoc_insertion_point(field_add:opencv_tensorflow.TensorProto.dcomplex_val)
}
inline const ::google::protobuf::RepeatedField<double>& TensorProto::dcomplex_val() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opencv_tensorflow.TensorProto.dcomplex_val)
  return _internal_dcomplex_val();
}
inline ::google::protobuf::RepeatedField<double>* PROTOBUF_NONNULL TensorProto::mutable_dcomplex_val()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable_list:opencv_tensorflow.TensorProto.dcomplex_val)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_dcomplex_val();
}
inline const ::google::protobuf::RepeatedField<double>&
TensorProto::_internal_dcomplex_val() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.dcomplex_val_;
}
inline ::google::protobuf::RepeatedField<double>* PROTOBUF_NONNULL
TensorProto::_internal_mutable_dcomplex_val() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.dcomplex_val_;
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace opencv_tensorflow


// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"

#endif  // tensor_2eproto_2epb_2eh
