/*M///////////////////////////////////////////////////////////////////////////////////////
//
//  IMPORTANT: READ BEFORE DOWNLOADING, COPYING, INSTALLING OR USING.
//
//  By downloading, copying, installing or using the software you agree to this license.
//  If you do not agree to this license, do not download, install,
//  copy or use the software.
//
//
//                           License Agreement
//                For Open Source Computer Vision Library
//
// Copyright (C) 2015, Itseez Inc, all rights reserved.
// Third party copyrights are property of their respective owners.
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//
//   * Redistribution's of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//
//   * Redistribution's in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//
//   * The name of the copyright holders may not be used to endorse or promote products
//     derived from this software without specific prior written permission.
//
// This software is provided by the copyright holders and contributors "as is" and
// any express or implied warranties, including, but not limited to, the implied
// warranties of merchantability and fitness for a particular purpose are disclaimed.
// In no event shall the Itseez Inc or contributors be liable for any direct,
// indirect, incidental, special, exemplary, or consequential damages
// (including, but not limited to, procurement of substitute goods or services;
// loss of use, data, or profits; or business interruption) however caused
// and on any theory of liability, whether in contract, strict liability,
// or tort (including negligence or otherwise) arising in any way out of
// the use of this software, even if advised of the possibility of such damage.
//
//M*/

#ifndef __DPM_CONVOLUTION__
#define __DPM_CONVOLUTION__

#include "opencv2/core.hpp"
#include <vector>

namespace cv
{
namespace dpm
{
/** @brief This class contains DPM model parameters
 */
class ConvolutionEngine
{
    public:
        // constructor
        ConvolutionEngine() {}

        // destructor
        ~ConvolutionEngine() {}

        // compute convolution value at a fixed location
        double convolve(const Mat &feat, const Mat &filter,
                int dimHOG, int x, int y);

        // compute convolution of a feature map and multiple filters
        // sum the filter convolution values into results
        void convolve(const Mat &feat, const Mat &filter,
                int dimHOG, Mat &result);
};
} // namespace dpm
} // namespace cv
#endif //__DPM_CONVOLUTION__
