/****************************************************************************
** Meta object code from reading C++ file 'window_QT.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../opencv/modules/highgui/src/window_QT.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'window_QT.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11GuiReceiverE_t {};
} // unnamed namespace

template <> constexpr inline auto GuiReceiver::qt_create_metaobjectdata<qt_meta_tag_ZN11GuiReceiverE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GuiReceiver",
        "createWindow",
        "",
        "name",
        "flags",
        "destroyWindow",
        "destroyAllWindow",
        "addSlider",
        "trackbar_name",
        "window_name",
        "value",
        "count",
        "on_change",
        "addSlider2",
        "userdata",
        "moveWindow",
        "x",
        "y",
        "resizeWindow",
        "width",
        "height",
        "showImage",
        "arr",
        "displayInfo",
        "text",
        "delayms",
        "displayStatusBar",
        "timeOut",
        "toggleFullScreen",
        "getWindowRect",
        "CvRect",
        "isFullScreen",
        "getPropWindow",
        "setPropWindow",
        "setWindowTitle",
        "title",
        "getWindowVisible",
        "getRatioWindow",
        "setRatioWindow",
        "arg2",
        "saveWindowParameters",
        "loadWindowParameters",
        "putText",
        "arg1",
        "QPoint",
        "org",
        "font",
        "addButton",
        "button_name",
        "button_type",
        "initial_button_state",
        "enablePropertiesButtonEachWindow",
        "setOpenGlDrawCallback",
        "callback",
        "setOpenGlContext",
        "updateWindow",
        "isOpenGl"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'createWindow'
        QtMocHelpers::SlotData<void(QString, int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Int, 4 },
        }}),
        // Slot 'createWindow'
        QtMocHelpers::SlotData<void(QString)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'destroyWindow'
        QtMocHelpers::SlotData<void(QString)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'destroyAllWindow'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'addSlider'
        QtMocHelpers::SlotData<void(QString, QString, void *, int, void *)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 }, { QMetaType::QString, 9 }, { QMetaType::VoidStar, 10 }, { QMetaType::Int, 11 },
            { QMetaType::VoidStar, 12 },
        }}),
        // Slot 'addSlider2'
        QtMocHelpers::SlotData<void(QString, QString, void *, int, void *, void *)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 }, { QMetaType::QString, 9 }, { QMetaType::VoidStar, 10 }, { QMetaType::Int, 11 },
            { QMetaType::VoidStar, 12 }, { QMetaType::VoidStar, 14 },
        }}),
        // Slot 'moveWindow'
        QtMocHelpers::SlotData<void(QString, int, int)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Int, 16 }, { QMetaType::Int, 17 },
        }}),
        // Slot 'resizeWindow'
        QtMocHelpers::SlotData<void(QString, int, int)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Int, 19 }, { QMetaType::Int, 20 },
        }}),
        // Slot 'showImage'
        QtMocHelpers::SlotData<void(QString, void *)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::VoidStar, 22 },
        }}),
        // Slot 'displayInfo'
        QtMocHelpers::SlotData<void(QString, QString, int)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 24 }, { QMetaType::Int, 25 },
        }}),
        // Slot 'displayStatusBar'
        QtMocHelpers::SlotData<void(QString, QString, int)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 24 }, { QMetaType::Int, 25 },
        }}),
        // Slot 'timeOut'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'toggleFullScreen'
        QtMocHelpers::SlotData<void(QString, double)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Double, 4 },
        }}),
        // Slot 'getWindowRect'
        QtMocHelpers::SlotData<CvRect(QString)>(29, 2, QMC::AccessPublic, 0x80000000 | 30, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'isFullScreen'
        QtMocHelpers::SlotData<double(QString)>(31, 2, QMC::AccessPublic, QMetaType::Double, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'getPropWindow'
        QtMocHelpers::SlotData<double(QString)>(32, 2, QMC::AccessPublic, QMetaType::Double, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'setPropWindow'
        QtMocHelpers::SlotData<void(QString, double)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Double, 4 },
        }}),
        // Slot 'setWindowTitle'
        QtMocHelpers::SlotData<void(QString, QString)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 35 },
        }}),
        // Slot 'getWindowVisible'
        QtMocHelpers::SlotData<double(QString)>(36, 2, QMC::AccessPublic, QMetaType::Double, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'getRatioWindow'
        QtMocHelpers::SlotData<double(QString)>(37, 2, QMC::AccessPublic, QMetaType::Double, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'setRatioWindow'
        QtMocHelpers::SlotData<void(QString, double)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Double, 39 },
        }}),
        // Slot 'saveWindowParameters'
        QtMocHelpers::SlotData<void(QString)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'loadWindowParameters'
        QtMocHelpers::SlotData<void(QString)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'putText'
        QtMocHelpers::SlotData<void(void *, QString, QPoint, void *)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::VoidStar, 43 }, { QMetaType::QString, 24 }, { 0x80000000 | 44, 45 }, { QMetaType::VoidStar, 46 },
        }}),
        // Slot 'addButton'
        QtMocHelpers::SlotData<void(QString, int, int, void *, void *)>(47, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 48 }, { QMetaType::Int, 49 }, { QMetaType::Int, 50 }, { QMetaType::VoidStar, 12 },
            { QMetaType::VoidStar, 14 },
        }}),
        // Slot 'enablePropertiesButtonEachWindow'
        QtMocHelpers::SlotData<void()>(51, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setOpenGlDrawCallback'
        QtMocHelpers::SlotData<void(QString, void *, void *)>(52, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::VoidStar, 53 }, { QMetaType::VoidStar, 14 },
        }}),
        // Slot 'setOpenGlContext'
        QtMocHelpers::SlotData<void(QString)>(54, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'updateWindow'
        QtMocHelpers::SlotData<void(QString)>(55, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'isOpenGl'
        QtMocHelpers::SlotData<double(QString)>(56, 2, QMC::AccessPublic, QMetaType::Double, {{
            { QMetaType::QString, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<GuiReceiver, qt_meta_tag_ZN11GuiReceiverE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GuiReceiver::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11GuiReceiverE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11GuiReceiverE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11GuiReceiverE_t>.metaTypes,
    nullptr
} };

void GuiReceiver::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<GuiReceiver *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->createWindow((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 1: _t->createWindow((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->destroyWindow((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->destroyAllWindow(); break;
        case 4: _t->addSlider((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<void*>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<void*>>(_a[5]))); break;
        case 5: _t->addSlider2((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<void*>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<void*>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<void*>>(_a[6]))); break;
        case 6: _t->moveWindow((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 7: _t->resizeWindow((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 8: _t->showImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<void*>>(_a[2]))); break;
        case 9: _t->displayInfo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 10: _t->displayStatusBar((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 11: _t->timeOut(); break;
        case 12: _t->toggleFullScreen((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 13: { CvRect _r = _t->getWindowRect((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<CvRect*>(_a[0]) = std::move(_r); }  break;
        case 14: { double _r = _t->isFullScreen((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 15: { double _r = _t->getPropWindow((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 16: _t->setPropWindow((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 17: _t->setWindowTitle((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 18: { double _r = _t->getWindowVisible((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 19: { double _r = _t->getRatioWindow((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 20: _t->setRatioWindow((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 21: _t->saveWindowParameters((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 22: _t->loadWindowParameters((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 23: _t->putText((*reinterpret_cast<std::add_pointer_t<void*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<void*>>(_a[4]))); break;
        case 24: _t->addButton((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<void*>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<void*>>(_a[5]))); break;
        case 25: _t->enablePropertiesButtonEachWindow(); break;
        case 26: _t->setOpenGlDrawCallback((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<void*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<void*>>(_a[3]))); break;
        case 27: _t->setOpenGlContext((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 28: _t->updateWindow((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 29: { double _r = _t->isOpenGl((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

const QMetaObject *GuiReceiver::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GuiReceiver::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11GuiReceiverE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int GuiReceiver::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 30)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 30;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 30)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 30;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN11CvButtonbarE_t {};
} // unnamed namespace

template <> constexpr inline auto CvButtonbar::qt_create_metaobjectdata<qt_meta_tag_ZN11CvButtonbarE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CvButtonbar"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CvButtonbar, qt_meta_tag_ZN11CvButtonbarE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CvButtonbar::staticMetaObject = { {
    QMetaObject::SuperData::link<CvBar::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11CvButtonbarE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11CvButtonbarE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11CvButtonbarE_t>.metaTypes,
    nullptr
} };

void CvButtonbar::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CvButtonbar *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *CvButtonbar::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CvButtonbar::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11CvButtonbarE_t>.strings))
        return static_cast<void*>(this);
    return CvBar::qt_metacast(_clname);
}

int CvButtonbar::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = CvBar::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN12CvPushButtonE_t {};
} // unnamed namespace

template <> constexpr inline auto CvPushButton::qt_create_metaobjectdata<qt_meta_tag_ZN12CvPushButtonE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CvPushButton",
        "callCallBack",
        ""
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'callCallBack'
        QtMocHelpers::SlotData<void(bool)>(1, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CvPushButton, qt_meta_tag_ZN12CvPushButtonE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CvPushButton::staticMetaObject = { {
    QMetaObject::SuperData::link<QPushButton::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12CvPushButtonE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12CvPushButtonE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12CvPushButtonE_t>.metaTypes,
    nullptr
} };

void CvPushButton::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CvPushButton *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->callCallBack((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *CvPushButton::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CvPushButton::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12CvPushButtonE_t>.strings))
        return static_cast<void*>(this);
    return QPushButton::qt_metacast(_clname);
}

int CvPushButton::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QPushButton::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN10CvCheckBoxE_t {};
} // unnamed namespace

template <> constexpr inline auto CvCheckBox::qt_create_metaobjectdata<qt_meta_tag_ZN10CvCheckBoxE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CvCheckBox",
        "callCallBack",
        ""
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'callCallBack'
        QtMocHelpers::SlotData<void(bool)>(1, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CvCheckBox, qt_meta_tag_ZN10CvCheckBoxE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CvCheckBox::staticMetaObject = { {
    QMetaObject::SuperData::link<QCheckBox::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10CvCheckBoxE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10CvCheckBoxE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10CvCheckBoxE_t>.metaTypes,
    nullptr
} };

void CvCheckBox::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CvCheckBox *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->callCallBack((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *CvCheckBox::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CvCheckBox::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10CvCheckBoxE_t>.strings))
        return static_cast<void*>(this);
    return QCheckBox::qt_metacast(_clname);
}

int CvCheckBox::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QCheckBox::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN13CvRadioButtonE_t {};
} // unnamed namespace

template <> constexpr inline auto CvRadioButton::qt_create_metaobjectdata<qt_meta_tag_ZN13CvRadioButtonE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CvRadioButton",
        "callCallBack",
        ""
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'callCallBack'
        QtMocHelpers::SlotData<void(bool)>(1, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CvRadioButton, qt_meta_tag_ZN13CvRadioButtonE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CvRadioButton::staticMetaObject = { {
    QMetaObject::SuperData::link<QRadioButton::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13CvRadioButtonE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13CvRadioButtonE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13CvRadioButtonE_t>.metaTypes,
    nullptr
} };

void CvRadioButton::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CvRadioButton *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->callCallBack((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *CvRadioButton::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CvRadioButton::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13CvRadioButtonE_t>.strings))
        return static_cast<void*>(this);
    return QRadioButton::qt_metacast(_clname);
}

int CvRadioButton::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QRadioButton::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN10CvTrackbarE_t {};
} // unnamed namespace

template <> constexpr inline auto CvTrackbar::qt_create_metaobjectdata<qt_meta_tag_ZN10CvTrackbarE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CvTrackbar",
        "createDialog",
        "",
        "update",
        "myvalue"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'createDialog'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'update'
        QtMocHelpers::SlotData<void(int)>(3, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CvTrackbar, qt_meta_tag_ZN10CvTrackbarE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CvTrackbar::staticMetaObject = { {
    QMetaObject::SuperData::link<CvBar::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10CvTrackbarE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10CvTrackbarE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10CvTrackbarE_t>.metaTypes,
    nullptr
} };

void CvTrackbar::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CvTrackbar *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->createDialog(); break;
        case 1: _t->update((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *CvTrackbar::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CvTrackbar::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10CvTrackbarE_t>.strings))
        return static_cast<void*>(this);
    return CvBar::qt_metacast(_clname);
}

int CvTrackbar::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = CvBar::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN15CvWinPropertiesE_t {};
} // unnamed namespace

template <> constexpr inline auto CvWinProperties::qt_create_metaobjectdata<qt_meta_tag_ZN15CvWinPropertiesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CvWinProperties"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CvWinProperties, qt_meta_tag_ZN15CvWinPropertiesE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CvWinProperties::staticMetaObject = { {
    QMetaObject::SuperData::link<CvWinModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15CvWinPropertiesE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15CvWinPropertiesE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15CvWinPropertiesE_t>.metaTypes,
    nullptr
} };

void CvWinProperties::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CvWinProperties *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *CvWinProperties::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CvWinProperties::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15CvWinPropertiesE_t>.strings))
        return static_cast<void*>(this);
    return CvWinModel::qt_metacast(_clname);
}

int CvWinProperties::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = CvWinModel::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN8CvWindowE_t {};
} // unnamed namespace

template <> constexpr inline auto CvWindow::qt_create_metaobjectdata<qt_meta_tag_ZN8CvWindowE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CvWindow",
        "displayPropertiesWin",
        ""
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'displayPropertiesWin'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CvWindow, qt_meta_tag_ZN8CvWindowE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CvWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<CvWinModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8CvWindowE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8CvWindowE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8CvWindowE_t>.metaTypes,
    nullptr
} };

void CvWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CvWindow *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->displayPropertiesWin(); break;
        default: ;
        }
    }
    (void)_a;
}

const QMetaObject *CvWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CvWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8CvWindowE_t>.strings))
        return static_cast<void*>(this);
    return CvWinModel::qt_metacast(_clname);
}

int CvWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = CvWinModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN15DefaultViewPortE_t {};
} // unnamed namespace

template <> constexpr inline auto DefaultViewPort::qt_create_metaobjectdata<qt_meta_tag_ZN15DefaultViewPortE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DefaultViewPort",
        "siftWindowOnLeft",
        "",
        "siftWindowOnRight",
        "siftWindowOnUp",
        "siftWindowOnDown",
        "resetZoom",
        "imgRegion",
        "ZoomIn",
        "ZoomOut",
        "saveView",
        "copy2Clipbrd",
        "stopDisplayInfo"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'siftWindowOnLeft'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'siftWindowOnRight'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'siftWindowOnUp'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'siftWindowOnDown'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'resetZoom'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'imgRegion'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ZoomIn'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ZoomOut'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'saveView'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'copy2Clipbrd'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'stopDisplayInfo'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DefaultViewPort, qt_meta_tag_ZN15DefaultViewPortE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DefaultViewPort::staticMetaObject = { {
    QMetaObject::SuperData::link<QGraphicsView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DefaultViewPortE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DefaultViewPortE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15DefaultViewPortE_t>.metaTypes,
    nullptr
} };

void DefaultViewPort::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DefaultViewPort *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->siftWindowOnLeft(); break;
        case 1: _t->siftWindowOnRight(); break;
        case 2: _t->siftWindowOnUp(); break;
        case 3: _t->siftWindowOnDown(); break;
        case 4: _t->resetZoom(); break;
        case 5: _t->imgRegion(); break;
        case 6: _t->ZoomIn(); break;
        case 7: _t->ZoomOut(); break;
        case 8: _t->saveView(); break;
        case 9: _t->copy2Clipbrd(); break;
        case 10: _t->stopDisplayInfo(); break;
        default: ;
        }
    }
    (void)_a;
}

const QMetaObject *DefaultViewPort::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DefaultViewPort::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DefaultViewPortE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "OCVViewPort"))
        return static_cast< OCVViewPort*>(this);
    return QGraphicsView::qt_metacast(_clname);
}

int DefaultViewPort::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QGraphicsView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}
QT_WARNING_POP
