"\n"
"General configuration for OpenCV 4.12.0 =====================================\n"
"  Version control:               unknown\n"
"\n"
"  Extra modules:\n"
"    Location (extra):            /build/opencv/src/opencv_contrib/modules\n"
"    Version control (extra):     unknown\n"
"\n"
"  Platform:\n"
"    Timestamp:                   2025-10-20T15:34:14Z\n"
"    Host:                        Linux 6.7.4-gnu-1 x86_64\n"
"    CMake:                       4.1.2\n"
"    CMake generator:             Unix Makefiles\n"
"    CMake build tool:            /usr/bin/make\n"
"    Configuration:               Release\n"
"    Algorithm Hint:              ALGO_HINT_ACCURATE\n"
"\n"
"  CPU/HW features:\n"
"    Baseline:                    SSE SSE2\n"
"      requested:                 SSE3\n"
"      required:                  SSE2\n"
"      disabled:                  SSE3\n"
"    Dispatched code generation:  SSE4_1 SSE4_2 AVX FP16 AVX2 AVX512_SKX\n"
"      SSE4_1 (15 files):         + SSE3 SSSE3 SSE4_1\n"
"      SSE4_2 (1 files):          + SSE3 SSSE3 SSE4_1 POPCNT SSE4_2\n"
"      AVX (9 files):             + SSE3 SSSE3 SSE4_1 POPCNT SSE4_2 AVX\n"
"      FP16 (0 files):            + SSE3 SSSE3 SSE4_1 POPCNT SSE4_2 AVX FP16\n"
"      AVX2 (35 files):           + SSE3 SSSE3 SSE4_1 POPCNT SSE4_2 AVX FP16 AVX2 FMA3\n"
"      AVX512_SKX (6 files):      + SSE3 SSSE3 SSE4_1 POPCNT SSE4_2 AVX FP16 AVX2 FMA3 AVX_512F AVX512_COMMON AVX512_SKX\n"
"\n"
"  C/C++:\n"
"    Built as dynamic libs?:      YES\n"
"    C++ standard:                17\n"
"    C++ Compiler:                /usr/bin/c++  (ver 15.2.1)\n"
"    C++ flags (Release):         -march=x86-64 -mtune=generic -O2 -pipe -fno-plt -fexceptions         -Wp,-D_FORTIFY_SOURCE=3 -Wformat -Werror=format-security         -fstack-clash-protection -fcf-protection         -fno-omit-frame-pointer -mno-omit-leaf-frame-pointer -Wp,-D_GLIBCXX_ASSERTIONS -g -ffile-prefix-map=/build/opencv/src=/usr/src/debug/opencv   -fsigned-char -W -Wall -Wreturn-type -Wnon-virtual-dtor -Waddress -Wsequence-point -Wformat -Wformat-security -Wmissing-declarations -Wundef -Winit-self -Wpointer-arith -Wshadow -Wsign-promo -Wuninitialized -Wsuggest-override -Wno-delete-non-virtual-dtor -Wno-comment -Wimplicit-fallthrough=3 -Wno-strict-overflow -fdiagnostics-show-option -pthread -fomit-frame-pointer -ffunction-sections -fdata-sections  -fvisibility=hidden -fvisibility-inlines-hidden -O3 -DNDEBUG  -DNDEBUG\n"
"    C++ flags (Debug):           -march=x86-64 -mtune=generic -O2 -pipe -fno-plt -fexceptions         -Wp,-D_FORTIFY_SOURCE=3 -Wformat -Werror=format-security         -fstack-clash-protection -fcf-protection         -fno-omit-frame-pointer -mno-omit-leaf-frame-pointer -Wp,-D_GLIBCXX_ASSERTIONS -g -ffile-prefix-map=/build/opencv/src=/usr/src/debug/opencv   -fsigned-char -W -Wall -Wreturn-type -Wnon-virtual-dtor -Waddress -Wsequence-point -Wformat -Wformat-security -Wmissing-declarations -Wundef -Winit-self -Wpointer-arith -Wshadow -Wsign-promo -Wuninitialized -Wsuggest-override -Wno-delete-non-virtual-dtor -Wno-comment -Wimplicit-fallthrough=3 -Wno-strict-overflow -fdiagnostics-show-option -pthread -fomit-frame-pointer -ffunction-sections -fdata-sections  -fvisibility=hidden -fvisibility-inlines-hidden -g  -DDEBUG -D_DEBUG\n"
"    C Compiler:                  /usr/bin/cc\n"
"    C flags (Release):           -march=x86-64 -mtune=generic -O2 -pipe -fno-plt -fexceptions         -Wp,-D_FORTIFY_SOURCE=3 -Wformat -Werror=format-security         -fstack-clash-protection -fcf-protection         -fno-omit-frame-pointer -mno-omit-leaf-frame-pointer -g -ffile-prefix-map=/build/opencv/src=/usr/src/debug/opencv   -fsigned-char -W -Wall -Wreturn-type -Waddress -Wsequence-point -Wformat -Wformat-security -Wmissing-declarations -Wmissing-prototypes -Wstrict-prototypes -Wundef -Winit-self -Wpointer-arith -Wshadow -Wuninitialized -Wno-comment -Wimplicit-fallthrough=3 -Wno-strict-overflow -fdiagnostics-show-option -pthread -fomit-frame-pointer -ffunction-sections -fdata-sections  -fvisibility=hidden -O3 -DNDEBUG  -DNDEBUG\n"
"    C flags (Debug):             -march=x86-64 -mtune=generic -O2 -pipe -fno-plt -fexceptions         -Wp,-D_FORTIFY_SOURCE=3 -Wformat -Werror=format-security         -fstack-clash-protection -fcf-protection         -fno-omit-frame-pointer -mno-omit-leaf-frame-pointer -g -ffile-prefix-map=/build/opencv/src=/usr/src/debug/opencv   -fsigned-char -W -Wall -Wreturn-type -Waddress -Wsequence-point -Wformat -Wformat-security -Wmissing-declarations -Wmissing-prototypes -Wstrict-prototypes -Wundef -Winit-self -Wpointer-arith -Wshadow -Wuninitialized -Wno-comment -Wimplicit-fallthrough=3 -Wno-strict-overflow -fdiagnostics-show-option -pthread -fomit-frame-pointer -ffunction-sections -fdata-sections  -fvisibility=hidden -g  -DDEBUG -D_DEBUG\n"
"    Linker flags (Release):      -Wl,-O1 -Wl,--sort-common -Wl,--as-needed -Wl,-z,relro -Wl,-z,now          -Wl,-z,pack-relative-relocs  -Wl,--gc-sections -Wl,--as-needed -Wl,--no-undefined  \n"
"    Linker flags (Debug):        -Wl,-O1 -Wl,--sort-common -Wl,--as-needed -Wl,-z,relro -Wl,-z,now          -Wl,-z,pack-relative-relocs  -Wl,--gc-sections -Wl,--as-needed -Wl,--no-undefined  \n"
"    ccache:                      NO\n"
"    Precompiled headers:         NO\n"
"    Extra dependencies:          dl m pthread rt\n"
"    3rdparty dependencies:\n"
"\n"
"  OpenCV modules:\n"
"    To be built:                 alphamat aruco bgsegm bioinspired calib3d ccalib core cvv datasets dnn dnn_objdetect dnn_superres dpm face features2d flann freetype fuzzy hdf hfs highgui img_hash imgcodecs imgproc intensity_transform java line_descriptor mcc ml objdetect optflow phase_unwrapping photo plot python3 quality rapid reg rgbd saliency shape signal stereo stitching structured_light superres surface_matching text tracking video videoio videostab viz wechat_qrcode ximgproc xobjdetect xphoto\n"
"    Disabled:                    world\n"
"    Disabled by dependency:      -\n"
"    Unavailable:                 cannops cudev fastcv gapi julia matlab ovis python2 sfm ts\n"
"    Applications:                examples apps\n"
"    Documentation:               NO\n"
"    Non-free algorithms:         YES\n"
"\n"
"  GUI:                           QT6\n"
"    QT:                          YES (ver 6.10.0 )\n"
"      QT OpenGL support:         YES (Qt6::OpenGL )\n"
"    GTK+:                        NO\n"
"    OpenGL support:              YES (/usr/lib/libGL.so /usr/lib/libGLU.so)\n"
"    VTK support:                 YES (ver 9.5.2)\n"
"\n"
"  Media I/O: \n"
"    ZLib:                        /usr/lib/libz.so (ver 1.3.1)\n"
"    JPEG:                        /usr/lib/libjpeg.so (ver 80)\n"
"    WEBP:                        /usr/lib/libwebp.so (ver decoder: 0x0210, encoder: 0x0210, demux: 0x0107)\n"
"    AVIF:                        NO\n"
"    PNG:                         /usr/lib/libpng.so (ver 1.6.50)\n"
"    TIFF:                        (ver 42 / 4.7.1)\n"
"    JPEG XL:                     /usr/lib/libjxl.so /usr/lib/libjxl_cms.so /usr/lib/liblcms2.so /usr/lib/libjxl_threads.so /usr/lib/libbrotlidec.so /usr/lib/libbrotlienc.so /usr/lib/libbrotlicommon.so /usr/lib/libhwy.so (ver 0.11.1)\n"
"    JPEG 2000:                   OpenJPEG (ver 2.5.4)\n"
"    OpenEXR:                     OpenEXR::OpenEXR (ver 3.4.2)\n"
"    GIF:                         YES\n"
"    HDR:                         YES\n"
"    SUNRASTER:                   YES\n"
"    PXM:                         YES\n"
"    PFM:                         YES\n"
"\n"
"  Video I/O:\n"
"    FFMPEG:                      YES\n"
"      avcodec:                   YES (62.11.100)\n"
"      avformat:                  YES (62.3.100)\n"
"      avutil:                    YES (60.8.100)\n"
"      swscale:                   YES (9.1.100)\n"
"      avresample:                NO\n"
"    GStreamer:                   YES (1.26.0)\n"
"    v4l/v4l2:                    YES (linux/videodev2.h)\n"
"\n"
"  Parallel framework:            TBB (ver 2022.2 interface 12160)\n"
"\n"
"  Trace:                         YES (with Intel ITT(3.25.4))\n"
"\n"
"  Other third-party libraries:\n"
"    VA:                          YES\n"
"    Lapack:                      YES (/usr/lib/liblapack.so /usr/lib/libblas.so /usr/lib/libcblas.so)\n"
"    Eigen:                       YES (ver 5.0.0)\n"
"    Custom HAL:                  NO\n"
"    Protobuf:                    /usr/lib/libprotobuf.so.32.1.0 (32.1.0)\n"
"    Flatbuffers:                 builtin/3rdparty (23.5.9)\n"
"\n"
"  Vulkan:                        YES\n"
"    Include path:                /build/opencv/src/opencv/3rdparty/include\n"
"    Link libraries:              Dynamic load\n"
"\n"
"  OpenCL:                        YES (INTELVA)\n"
"    Include path:                /build/opencv/src/opencv/3rdparty/include/opencl/1.2\n"
"    Link libraries:              Dynamic load\n"
"\n"
"  Python 3:\n"
"    Interpreter:                 /usr/bin/python3 (ver 3.13.7)\n"
"    Libraries:                   /usr/lib/libpython3.13.so (ver 3.13.7)\n"
"    Limited API:                 NO\n"
"    numpy:                       /usr/lib/python3.13/site-packages/numpy/_core/include (ver 2.3.4)\n"
"    install path:                lib/python3.13/site-packages\n"
"\n"
"  Python (for build):            /usr/bin/python3\n"
"\n"
"  Java:                          \n"
"    ant:                         /bin/ant (ver 1.10.15)\n"
"    Java:                        NO\n"
"    JNI:                         /usr/lib/jvm/default/include /usr/lib/jvm/default/include/linux /usr/lib/jvm/default/include\n"
"    Java wrappers:               YES (ANT)\n"
"    Java tests:                  NO\n"
"\n"
"  Install to:                    /usr\n"
"-----------------------------------------------------------------\n"
"\n"
