/****************************************************************************
** Meta object code from reading C++ file 'keypointmanagement.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../opencv_contrib/modules/cvv/src/qtutil/matchview/keypointmanagement.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'keypointmanagement.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3cvv6qtutil18KeyPointManagementE_t {};
} // unnamed namespace

template <> constexpr inline auto cvv::qtutil::KeyPointManagement::qt_create_metaobjectdata<qt_meta_tag_ZN3cvv6qtutil18KeyPointManagementE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "cvv::qtutil::KeyPointManagement",
        "updateSelection",
        "",
        "std::vector<cv::KeyPoint>",
        "selection",
        "applySettingsToSelection",
        "KeyPointSettings&",
        "addToSelection",
        "cv::KeyPoint",
        "key",
        "singleSelection",
        "setSelection",
        "addSetting",
        "removeSetting",
        "KeyPointSettingsSelector*",
        "setting",
        "addSelection",
        "removeSelection",
        "KeyPointSelectionSelector*",
        "selector",
        "applySelection",
        "selectAll",
        "selectNone"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'updateSelection'
        QtMocHelpers::SignalData<void(const std::vector<cv::KeyPoint> &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'applySettingsToSelection'
        QtMocHelpers::SignalData<void(KeyPointSettings &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 2 },
        }}),
        // Slot 'addToSelection'
        QtMocHelpers::SlotData<void(const cv::KeyPoint &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'singleSelection'
        QtMocHelpers::SlotData<void(const cv::KeyPoint &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'setSelection'
        QtMocHelpers::SlotData<void(const std::vector<cv::KeyPoint> &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'addSetting'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'removeSetting'
        QtMocHelpers::SlotData<void(KeyPointSettingsSelector *)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'addSelection'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'removeSelection'
        QtMocHelpers::SlotData<void(KeyPointSelectionSelector *)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Slot 'applySelection'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectAll'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectNone'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KeyPointManagement, qt_meta_tag_ZN3cvv6qtutil18KeyPointManagementE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject cvv::qtutil::KeyPointManagement::staticMetaObject = { {
    QMetaObject::SuperData::link<KeyPointSettings::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3cvv6qtutil18KeyPointManagementE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3cvv6qtutil18KeyPointManagementE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3cvv6qtutil18KeyPointManagementE_t>.metaTypes,
    nullptr
} };

void cvv::qtutil::KeyPointManagement::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KeyPointManagement *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->updateSelection((*reinterpret_cast<std::add_pointer_t<std::vector<cv::KeyPoint>>>(_a[1]))); break;
        case 1: _t->applySettingsToSelection((*reinterpret_cast<std::add_pointer_t<KeyPointSettings&>>(_a[1]))); break;
        case 2: _t->addToSelection((*reinterpret_cast<std::add_pointer_t<cv::KeyPoint>>(_a[1]))); break;
        case 3: _t->singleSelection((*reinterpret_cast<std::add_pointer_t<cv::KeyPoint>>(_a[1]))); break;
        case 4: _t->setSelection((*reinterpret_cast<std::add_pointer_t<std::vector<cv::KeyPoint>>>(_a[1]))); break;
        case 5: _t->addSetting(); break;
        case 6: _t->removeSetting((*reinterpret_cast<std::add_pointer_t<KeyPointSettingsSelector*>>(_a[1]))); break;
        case 7: _t->addSelection(); break;
        case 8: _t->removeSelection((*reinterpret_cast<std::add_pointer_t<KeyPointSelectionSelector*>>(_a[1]))); break;
        case 9: _t->applySelection(); break;
        case 10: _t->selectAll(); break;
        case 11: _t->selectNone(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KeyPointSettingsSelector* >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KeyPointSelectionSelector* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KeyPointManagement::*)(const std::vector<cv::KeyPoint> & )>(_a, &KeyPointManagement::updateSelection, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KeyPointManagement::*)(KeyPointSettings & )>(_a, &KeyPointManagement::applySettingsToSelection, 1))
            return;
    }
}

const QMetaObject *cvv::qtutil::KeyPointManagement::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *cvv::qtutil::KeyPointManagement::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3cvv6qtutil18KeyPointManagementE_t>.strings))
        return static_cast<void*>(this);
    return KeyPointSettings::qt_metacast(_clname);
}

int cvv::qtutil::KeyPointManagement::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KeyPointSettings::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void cvv::qtutil::KeyPointManagement::updateSelection(const std::vector<cv::KeyPoint> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void cvv::qtutil::KeyPointManagement::applySettingsToSelection(KeyPointSettings & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
QT_WARNING_POP
