/****************************************************************************
** Meta object code from reading C++ file 'zoomableimage.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../opencv_contrib/modules/cvv/src/qtutil/zoomableimage.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'zoomableimage.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3cvv6qtutil13ZoomableImageE_t {};
} // unnamed namespace

template <> constexpr inline auto cvv::qtutil::ZoomableImage::qt_create_metaobjectdata<qt_meta_tag_ZN3cvv6qtutil13ZoomableImageE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "cvv::qtutil::ZoomableImage",
        "updateConversionResult",
        "",
        "cv::Mat",
        "ImageConversionResult",
        "updateArea",
        "QRectF",
        "updateMouseHover",
        "QPointF",
        "setArea",
        "rect",
        "zoom",
        "setMatR",
        "cv::Mat&",
        "mat",
        "setMat",
        "setZoom",
        "factor",
        "setAutoShowValues",
        "enable",
        "setThreshold",
        "threshold",
        "showFullImage",
        "setCTRLZoomFactor",
        "setCTRLShiftZoomFactor",
        "wheelEvent",
        "QWheelEvent*",
        "event",
        "setUpdateAreaDelay",
        "i",
        "viewScrolled",
        "drawValues",
        "rightClick",
        "QPoint",
        "pos",
        "emitUpdateArea",
        "queueUpdateArea"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'updateConversionResult'
        QtMocHelpers::SignalData<void(const cv::Mat &, ImageConversionResult) const>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { 0x80000000 | 4, 2 },
        }}),
        // Signal 'updateArea'
        QtMocHelpers::SignalData<void(QRectF, qreal) const>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 2 }, { QMetaType::QReal, 2 },
        }}),
        // Signal 'updateMouseHover'
        QtMocHelpers::SignalData<void(QPointF, QString, bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 2 }, { QMetaType::QString, 2 }, { QMetaType::Bool, 2 },
        }}),
        // Slot 'setArea'
        QtMocHelpers::SlotData<void(QRectF, qreal)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 10 }, { QMetaType::QReal, 11 },
        }}),
        // Slot 'setMatR'
        QtMocHelpers::SlotData<void(cv::Mat &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Slot 'setMat'
        QtMocHelpers::SlotData<void(cv::Mat)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 14 },
        }}),
        // Slot 'setZoom'
        QtMocHelpers::SlotData<void(qreal)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 17 },
        }}),
        // Slot 'setZoom'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'setAutoShowValues'
        QtMocHelpers::SlotData<void(bool)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 19 },
        }}),
        // Slot 'setAutoShowValues'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'setThreshold'
        QtMocHelpers::SlotData<void(qreal)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 21 },
        }}),
        // Slot 'setThreshold'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'showFullImage'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setCTRLZoomFactor'
        QtMocHelpers::SlotData<void(qreal)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 17 },
        }}),
        // Slot 'setCTRLZoomFactor'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'setCTRLShiftZoomFactor'
        QtMocHelpers::SlotData<void(qreal)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 17 },
        }}),
        // Slot 'setCTRLShiftZoomFactor'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'wheelEvent'
        QtMocHelpers::SlotData<void(QWheelEvent *)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 26, 27 },
        }}),
        // Slot 'setUpdateAreaDelay'
        QtMocHelpers::SlotData<void(int)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 29 },
        }}),
        // Slot 'viewScrolled'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'drawValues'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'rightClick'
        QtMocHelpers::SlotData<void(const QPoint &)>(32, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 33, 34 },
        }}),
        // Slot 'emitUpdateArea'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'queueUpdateArea'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ZoomableImage, qt_meta_tag_ZN3cvv6qtutil13ZoomableImageE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject cvv::qtutil::ZoomableImage::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3cvv6qtutil13ZoomableImageE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3cvv6qtutil13ZoomableImageE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3cvv6qtutil13ZoomableImageE_t>.metaTypes,
    nullptr
} };

void cvv::qtutil::ZoomableImage::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ZoomableImage *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->updateConversionResult((*reinterpret_cast<std::add_pointer_t<cv::Mat>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ImageConversionResult>>(_a[2]))); break;
        case 1: _t->updateArea((*reinterpret_cast<std::add_pointer_t<QRectF>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2]))); break;
        case 2: _t->updateMouseHover((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 3: _t->setArea((*reinterpret_cast<std::add_pointer_t<QRectF>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2]))); break;
        case 4: _t->setMatR((*reinterpret_cast<std::add_pointer_t<cv::Mat&>>(_a[1]))); break;
        case 5: _t->setMat((*reinterpret_cast<std::add_pointer_t<cv::Mat>>(_a[1]))); break;
        case 6: _t->setZoom((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 7: _t->setZoom(); break;
        case 8: _t->setAutoShowValues((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->setAutoShowValues(); break;
        case 10: _t->setThreshold((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 11: _t->setThreshold(); break;
        case 12: _t->showFullImage(); break;
        case 13: _t->setCTRLZoomFactor((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 14: _t->setCTRLZoomFactor(); break;
        case 15: _t->setCTRLShiftZoomFactor((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 16: _t->setCTRLShiftZoomFactor(); break;
        case 17: _t->wheelEvent((*reinterpret_cast<std::add_pointer_t<QWheelEvent*>>(_a[1]))); break;
        case 18: _t->setUpdateAreaDelay((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 19: _t->viewScrolled(); break;
        case 20: _t->drawValues(); break;
        case 21: _t->rightClick((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 22: _t->emitUpdateArea(); break;
        case 23: _t->queueUpdateArea(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ZoomableImage::*)(const cv::Mat & , ImageConversionResult ) const>(_a, &ZoomableImage::updateConversionResult, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ZoomableImage::*)(QRectF , qreal ) const>(_a, &ZoomableImage::updateArea, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ZoomableImage::*)(QPointF , QString , bool )>(_a, &ZoomableImage::updateMouseHover, 2))
            return;
    }
}

const QMetaObject *cvv::qtutil::ZoomableImage::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *cvv::qtutil::ZoomableImage::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3cvv6qtutil13ZoomableImageE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int cvv::qtutil::ZoomableImage::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 24)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 24;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 24)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 24;
    }
    return _id;
}

// SIGNAL 0
void cvv::qtutil::ZoomableImage::updateConversionResult(const cv::Mat & _t1, ImageConversionResult _t2)const
{
    QMetaObject::activate<void>(const_cast< cvv::qtutil::ZoomableImage *>(this), &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void cvv::qtutil::ZoomableImage::updateArea(QRectF _t1, qreal _t2)const
{
    QMetaObject::activate<void>(const_cast< cvv::qtutil::ZoomableImage *>(this), &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void cvv::qtutil::ZoomableImage::updateMouseHover(QPointF _t1, QString _t2, bool _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3);
}
QT_WARNING_POP
