// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: graph.proto
// Protobuf C++ Version: 6.32.1

#ifndef graph_2eproto_2epb_2eh
#define graph_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 6032001
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/message.h"
#include "google/protobuf/message_lite.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/map.h"  // IWYU pragma: export
#include "google/protobuf/map_type_handler.h"  // IWYU pragma: export
#include "google/protobuf/map_entry.h"
#include "google/protobuf/map_field.h"
#include "google/protobuf/unknown_field_set.h"
#include "attr_value.pb.h"
#include "function.pb.h"
#include "versions.pb.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_graph_2eproto

namespace google {
namespace protobuf {
namespace internal {
template <typename T>
::absl::string_view GetAnyMessageName();
}  // namespace internal
}  // namespace protobuf
}  // namespace google

// Internal implementation detail -- do not use these members.
struct TableStruct_graph_2eproto {
  static const ::uint32_t offsets[];
};
extern "C" {
extern const ::google::protobuf::internal::DescriptorTable descriptor_table_graph_2eproto;
}  // extern "C"
namespace opencv_tensorflow {
class GraphDef;
struct GraphDefDefaultTypeInternal;
extern GraphDefDefaultTypeInternal _GraphDef_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull GraphDef_class_data_;
class NodeDef;
struct NodeDefDefaultTypeInternal;
extern NodeDefDefaultTypeInternal _NodeDef_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull NodeDef_class_data_;
class NodeDef_AttrEntry_DoNotUse;
struct NodeDef_AttrEntry_DoNotUseDefaultTypeInternal;
extern NodeDef_AttrEntry_DoNotUseDefaultTypeInternal _NodeDef_AttrEntry_DoNotUse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull NodeDef_AttrEntry_DoNotUse_class_data_;
}  // namespace opencv_tensorflow
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google

namespace opencv_tensorflow {

// ===================================================================


// -------------------------------------------------------------------

class NodeDef_AttrEntry_DoNotUse final
    : public ::google::protobuf::internal::MapEntry<::std::string, ::google::protobuf::Message,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                             ::google::protobuf::internal::WireFormatLite::TYPE_MESSAGE> {
 public:
  using SuperType =
      ::google::protobuf::internal::MapEntry<::std::string, ::google::protobuf::Message,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_MESSAGE>;
  NodeDef_AttrEntry_DoNotUse();
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR NodeDef_AttrEntry_DoNotUse(::google::protobuf::internal::ConstantInitialized);
  explicit NodeDef_AttrEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr const void* PROTOBUF_NONNULL internal_default_instance() {
    return &_NodeDef_AttrEntry_DoNotUse_default_instance_;
  }


  static constexpr auto InternalGenerateClassData_();

 private:
  friend class ::google::protobuf::MessageLite;
  friend struct ::TableStruct_graph_2eproto;

  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   1, 47,
                                   2>
      _table_;

  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();
};
extern const ::google::protobuf::internal::ClassDataFull NodeDef_AttrEntry_DoNotUse_class_data_;
// -------------------------------------------------------------------

class NodeDef final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:opencv_tensorflow.NodeDef) */ {
 public:
  inline NodeDef() : NodeDef(nullptr) {}
  ~NodeDef() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(NodeDef* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(NodeDef));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR NodeDef(::google::protobuf::internal::ConstantInitialized);

  inline NodeDef(const NodeDef& from) : NodeDef(nullptr, from) {}
  inline NodeDef(NodeDef&& from) noexcept
      : NodeDef(nullptr, ::std::move(from)) {}
  inline NodeDef& operator=(const NodeDef& from) {
    CopyFrom(from);
    return *this;
  }
  inline NodeDef& operator=(NodeDef&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const NodeDef& default_instance() {
    return *reinterpret_cast<const NodeDef*>(
        &_NodeDef_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 2;
  friend void swap(NodeDef& a, NodeDef& b) { a.Swap(&b); }
  inline void Swap(NodeDef* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(NodeDef* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  NodeDef* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<NodeDef>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const NodeDef& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const NodeDef& from) { NodeDef::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(NodeDef* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "opencv_tensorflow.NodeDef"; }

 protected:
  explicit NodeDef(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  NodeDef(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const NodeDef& from);
  NodeDef(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, NodeDef&& from) noexcept
      : NodeDef(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kInputFieldNumber = 3,
    kAttrFieldNumber = 5,
    kNameFieldNumber = 1,
    kOpFieldNumber = 2,
    kDeviceFieldNumber = 4,
  };
  // repeated string input = 3;
  int input_size() const;
  private:
  int _internal_input_size() const;

  public:
  void clear_input() ;
  const ::std::string& input(int index) const;
  ::std::string* PROTOBUF_NONNULL mutable_input(int index);
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_input(int index, Arg_&& value, Args_... args);
  ::std::string* PROTOBUF_NONNULL add_input();
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void add_input(Arg_&& value, Args_... args);
  const ::google::protobuf::RepeatedPtrField<::std::string>& input() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL mutable_input();

  private:
  const ::google::protobuf::RepeatedPtrField<::std::string>& _internal_input() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL _internal_mutable_input();

  public:
  // map<string, .opencv_tensorflow.AttrValue> attr = 5;
  int attr_size() const;
  private:
  int _internal_attr_size() const;

  public:
  void clear_attr() ;
  const ::google::protobuf::Map<::std::string, ::opencv_tensorflow::AttrValue>& attr() const;
  ::google::protobuf::Map<::std::string, ::opencv_tensorflow::AttrValue>* PROTOBUF_NONNULL mutable_attr();

  private:
  const ::google::protobuf::Map<::std::string, ::opencv_tensorflow::AttrValue>& _internal_attr() const;
  ::google::protobuf::Map<::std::string, ::opencv_tensorflow::AttrValue>* PROTOBUF_NONNULL _internal_mutable_attr();

  public:
  // string name = 1;
  void clear_name() ;
  const ::std::string& name() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_name(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_name();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_name();
  void set_allocated_name(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_name() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_name(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_name();

  public:
  // string op = 2;
  void clear_op() ;
  const ::std::string& op() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_op(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_op();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_op();
  void set_allocated_op(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_op() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_op(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_op();

  public:
  // string device = 4;
  void clear_device() ;
  const ::std::string& device() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_device(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_device();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_device();
  void set_allocated_device(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_device() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_device(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_device();

  public:
  // @@protoc_insertion_point(class_scope:opencv_tensorflow.NodeDef)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<2, 5,
                                   2, 55,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const NodeDef& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField<::std::string> input_;
    ::google::protobuf::internal::MapField<NodeDef_AttrEntry_DoNotUse, ::std::string, ::opencv_tensorflow::AttrValue,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_MESSAGE>
        attr_;
    ::google::protobuf::internal::ArenaStringPtr name_;
    ::google::protobuf::internal::ArenaStringPtr op_;
    ::google::protobuf::internal::ArenaStringPtr device_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_graph_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull NodeDef_class_data_;
// -------------------------------------------------------------------

class GraphDef final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:opencv_tensorflow.GraphDef) */ {
 public:
  inline GraphDef() : GraphDef(nullptr) {}
  ~GraphDef() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(GraphDef* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(GraphDef));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR GraphDef(::google::protobuf::internal::ConstantInitialized);

  inline GraphDef(const GraphDef& from) : GraphDef(nullptr, from) {}
  inline GraphDef(GraphDef&& from) noexcept
      : GraphDef(nullptr, ::std::move(from)) {}
  inline GraphDef& operator=(const GraphDef& from) {
    CopyFrom(from);
    return *this;
  }
  inline GraphDef& operator=(GraphDef&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const GraphDef& default_instance() {
    return *reinterpret_cast<const GraphDef*>(
        &_GraphDef_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(GraphDef& a, GraphDef& b) { a.Swap(&b); }
  inline void Swap(GraphDef* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(GraphDef* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  GraphDef* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<GraphDef>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const GraphDef& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const GraphDef& from) { GraphDef::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(GraphDef* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "opencv_tensorflow.GraphDef"; }

 protected:
  explicit GraphDef(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  GraphDef(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const GraphDef& from);
  GraphDef(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, GraphDef&& from) noexcept
      : GraphDef(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kNodeFieldNumber = 1,
    kLibraryFieldNumber = 2,
    kVersionsFieldNumber = 4,
    kVersionFieldNumber = 3,
  };
  // repeated .opencv_tensorflow.NodeDef node = 1;
  int node_size() const;
  private:
  int _internal_node_size() const;

  public:
  void clear_node() ;
  ::opencv_tensorflow::NodeDef* PROTOBUF_NONNULL mutable_node(int index);
  ::google::protobuf::RepeatedPtrField<::opencv_tensorflow::NodeDef>* PROTOBUF_NONNULL mutable_node();

  private:
  const ::google::protobuf::RepeatedPtrField<::opencv_tensorflow::NodeDef>& _internal_node() const;
  ::google::protobuf::RepeatedPtrField<::opencv_tensorflow::NodeDef>* PROTOBUF_NONNULL _internal_mutable_node();
  public:
  const ::opencv_tensorflow::NodeDef& node(int index) const;
  ::opencv_tensorflow::NodeDef* PROTOBUF_NONNULL add_node();
  const ::google::protobuf::RepeatedPtrField<::opencv_tensorflow::NodeDef>& node() const;
  // .opencv_tensorflow.FunctionDefLibrary library = 2;
  bool has_library() const;
  void clear_library() ;
  const ::opencv_tensorflow::FunctionDefLibrary& library() const;
  [[nodiscard]] ::opencv_tensorflow::FunctionDefLibrary* PROTOBUF_NULLABLE release_library();
  ::opencv_tensorflow::FunctionDefLibrary* PROTOBUF_NONNULL mutable_library();
  void set_allocated_library(::opencv_tensorflow::FunctionDefLibrary* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_library(::opencv_tensorflow::FunctionDefLibrary* PROTOBUF_NULLABLE value);
  ::opencv_tensorflow::FunctionDefLibrary* PROTOBUF_NULLABLE unsafe_arena_release_library();

  private:
  const ::opencv_tensorflow::FunctionDefLibrary& _internal_library() const;
  ::opencv_tensorflow::FunctionDefLibrary* PROTOBUF_NONNULL _internal_mutable_library();

  public:
  // .opencv_tensorflow.VersionDef versions = 4;
  bool has_versions() const;
  void clear_versions() ;
  const ::opencv_tensorflow::VersionDef& versions() const;
  [[nodiscard]] ::opencv_tensorflow::VersionDef* PROTOBUF_NULLABLE release_versions();
  ::opencv_tensorflow::VersionDef* PROTOBUF_NONNULL mutable_versions();
  void set_allocated_versions(::opencv_tensorflow::VersionDef* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_versions(::opencv_tensorflow::VersionDef* PROTOBUF_NULLABLE value);
  ::opencv_tensorflow::VersionDef* PROTOBUF_NULLABLE unsafe_arena_release_versions();

  private:
  const ::opencv_tensorflow::VersionDef& _internal_versions() const;
  ::opencv_tensorflow::VersionDef* PROTOBUF_NONNULL _internal_mutable_versions();

  public:
  // int32 version = 3 [deprecated = true];
  [[deprecated]]  void clear_version() ;
  [[deprecated]] ::int32_t version() const;
  [[deprecated]] void set_version(::int32_t value);

  private:
  ::int32_t _internal_version() const;
  void _internal_set_version(::int32_t value);

  public:
  // @@protoc_insertion_point(class_scope:opencv_tensorflow.GraphDef)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<2, 4,
                                   3, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const GraphDef& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::opencv_tensorflow::NodeDef > node_;
    ::opencv_tensorflow::FunctionDefLibrary* PROTOBUF_NULLABLE library_;
    ::opencv_tensorflow::VersionDef* PROTOBUF_NULLABLE versions_;
    ::int32_t version_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_graph_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull GraphDef_class_data_;

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// GraphDef

// repeated .opencv_tensorflow.NodeDef node = 1;
inline int GraphDef::_internal_node_size() const {
  return _internal_node().size();
}
inline int GraphDef::node_size() const {
  return _internal_node_size();
}
inline void GraphDef::clear_node() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.node_.Clear();
}
inline ::opencv_tensorflow::NodeDef* PROTOBUF_NONNULL GraphDef::mutable_node(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:opencv_tensorflow.GraphDef.node)
  return _internal_mutable_node()->Mutable(index);
}
inline ::google::protobuf::RepeatedPtrField<::opencv_tensorflow::NodeDef>* PROTOBUF_NONNULL GraphDef::mutable_node()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable_list:opencv_tensorflow.GraphDef.node)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_node();
}
inline const ::opencv_tensorflow::NodeDef& GraphDef::node(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opencv_tensorflow.GraphDef.node)
  return _internal_node().Get(index);
}
inline ::opencv_tensorflow::NodeDef* PROTOBUF_NONNULL GraphDef::add_node()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::opencv_tensorflow::NodeDef* _add = _internal_mutable_node()->Add();
  // @@protoc_insertion_point(field_add:opencv_tensorflow.GraphDef.node)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::opencv_tensorflow::NodeDef>& GraphDef::node() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opencv_tensorflow.GraphDef.node)
  return _internal_node();
}
inline const ::google::protobuf::RepeatedPtrField<::opencv_tensorflow::NodeDef>&
GraphDef::_internal_node() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.node_;
}
inline ::google::protobuf::RepeatedPtrField<::opencv_tensorflow::NodeDef>* PROTOBUF_NONNULL
GraphDef::_internal_mutable_node() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.node_;
}

// .opencv_tensorflow.VersionDef versions = 4;
inline bool GraphDef::has_versions() const {
  bool value = (_impl_._has_bits_[0] & 0x00000002U) != 0;
  PROTOBUF_ASSUME(!value || _impl_.versions_ != nullptr);
  return value;
}
inline const ::opencv_tensorflow::VersionDef& GraphDef::_internal_versions() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::opencv_tensorflow::VersionDef* p = _impl_.versions_;
  return p != nullptr ? *p : reinterpret_cast<const ::opencv_tensorflow::VersionDef&>(::opencv_tensorflow::_VersionDef_default_instance_);
}
inline const ::opencv_tensorflow::VersionDef& GraphDef::versions() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opencv_tensorflow.GraphDef.versions)
  return _internal_versions();
}
inline void GraphDef::unsafe_arena_set_allocated_versions(
    ::opencv_tensorflow::VersionDef* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.versions_);
  }
  _impl_.versions_ = reinterpret_cast<::opencv_tensorflow::VersionDef*>(value);
  if (value != nullptr) {
    _impl_._has_bits_[0] |= 0x00000002U;
  } else {
    _impl_._has_bits_[0] &= ~0x00000002U;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:opencv_tensorflow.GraphDef.versions)
}
inline ::opencv_tensorflow::VersionDef* PROTOBUF_NULLABLE GraphDef::release_versions() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  _impl_._has_bits_[0] &= ~0x00000002U;
  ::opencv_tensorflow::VersionDef* released = _impl_.versions_;
  _impl_.versions_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::opencv_tensorflow::VersionDef* PROTOBUF_NULLABLE GraphDef::unsafe_arena_release_versions() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:opencv_tensorflow.GraphDef.versions)

  _impl_._has_bits_[0] &= ~0x00000002U;
  ::opencv_tensorflow::VersionDef* temp = _impl_.versions_;
  _impl_.versions_ = nullptr;
  return temp;
}
inline ::opencv_tensorflow::VersionDef* PROTOBUF_NONNULL GraphDef::_internal_mutable_versions() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.versions_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::opencv_tensorflow::VersionDef>(GetArena());
    _impl_.versions_ = reinterpret_cast<::opencv_tensorflow::VersionDef*>(p);
  }
  return _impl_.versions_;
}
inline ::opencv_tensorflow::VersionDef* PROTOBUF_NONNULL GraphDef::mutable_versions()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  _impl_._has_bits_[0] |= 0x00000002U;
  ::opencv_tensorflow::VersionDef* _msg = _internal_mutable_versions();
  // @@protoc_insertion_point(field_mutable:opencv_tensorflow.GraphDef.versions)
  return _msg;
}
inline void GraphDef::set_allocated_versions(::opencv_tensorflow::VersionDef* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.versions_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    _impl_._has_bits_[0] |= 0x00000002U;
  } else {
    _impl_._has_bits_[0] &= ~0x00000002U;
  }

  _impl_.versions_ = reinterpret_cast<::opencv_tensorflow::VersionDef*>(value);
  // @@protoc_insertion_point(field_set_allocated:opencv_tensorflow.GraphDef.versions)
}

// int32 version = 3 [deprecated = true];
inline void GraphDef::clear_version() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.version_ = 0;
  _impl_._has_bits_[0] &= ~0x00000004U;
}
inline ::int32_t GraphDef::version() const {
  // @@protoc_insertion_point(field_get:opencv_tensorflow.GraphDef.version)
  return _internal_version();
}
inline void GraphDef::set_version(::int32_t value) {
  _internal_set_version(value);
  _impl_._has_bits_[0] |= 0x00000004U;
  // @@protoc_insertion_point(field_set:opencv_tensorflow.GraphDef.version)
}
inline ::int32_t GraphDef::_internal_version() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.version_;
}
inline void GraphDef::_internal_set_version(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.version_ = value;
}

// .opencv_tensorflow.FunctionDefLibrary library = 2;
inline bool GraphDef::has_library() const {
  bool value = (_impl_._has_bits_[0] & 0x00000001U) != 0;
  PROTOBUF_ASSUME(!value || _impl_.library_ != nullptr);
  return value;
}
inline const ::opencv_tensorflow::FunctionDefLibrary& GraphDef::_internal_library() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::opencv_tensorflow::FunctionDefLibrary* p = _impl_.library_;
  return p != nullptr ? *p : reinterpret_cast<const ::opencv_tensorflow::FunctionDefLibrary&>(::opencv_tensorflow::_FunctionDefLibrary_default_instance_);
}
inline const ::opencv_tensorflow::FunctionDefLibrary& GraphDef::library() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opencv_tensorflow.GraphDef.library)
  return _internal_library();
}
inline void GraphDef::unsafe_arena_set_allocated_library(
    ::opencv_tensorflow::FunctionDefLibrary* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.library_);
  }
  _impl_.library_ = reinterpret_cast<::opencv_tensorflow::FunctionDefLibrary*>(value);
  if (value != nullptr) {
    _impl_._has_bits_[0] |= 0x00000001U;
  } else {
    _impl_._has_bits_[0] &= ~0x00000001U;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:opencv_tensorflow.GraphDef.library)
}
inline ::opencv_tensorflow::FunctionDefLibrary* PROTOBUF_NULLABLE GraphDef::release_library() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  _impl_._has_bits_[0] &= ~0x00000001U;
  ::opencv_tensorflow::FunctionDefLibrary* released = _impl_.library_;
  _impl_.library_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::opencv_tensorflow::FunctionDefLibrary* PROTOBUF_NULLABLE GraphDef::unsafe_arena_release_library() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:opencv_tensorflow.GraphDef.library)

  _impl_._has_bits_[0] &= ~0x00000001U;
  ::opencv_tensorflow::FunctionDefLibrary* temp = _impl_.library_;
  _impl_.library_ = nullptr;
  return temp;
}
inline ::opencv_tensorflow::FunctionDefLibrary* PROTOBUF_NONNULL GraphDef::_internal_mutable_library() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.library_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::opencv_tensorflow::FunctionDefLibrary>(GetArena());
    _impl_.library_ = reinterpret_cast<::opencv_tensorflow::FunctionDefLibrary*>(p);
  }
  return _impl_.library_;
}
inline ::opencv_tensorflow::FunctionDefLibrary* PROTOBUF_NONNULL GraphDef::mutable_library()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  _impl_._has_bits_[0] |= 0x00000001U;
  ::opencv_tensorflow::FunctionDefLibrary* _msg = _internal_mutable_library();
  // @@protoc_insertion_point(field_mutable:opencv_tensorflow.GraphDef.library)
  return _msg;
}
inline void GraphDef::set_allocated_library(::opencv_tensorflow::FunctionDefLibrary* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.library_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    _impl_._has_bits_[0] |= 0x00000001U;
  } else {
    _impl_._has_bits_[0] &= ~0x00000001U;
  }

  _impl_.library_ = reinterpret_cast<::opencv_tensorflow::FunctionDefLibrary*>(value);
  // @@protoc_insertion_point(field_set_allocated:opencv_tensorflow.GraphDef.library)
}

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// NodeDef

// string name = 1;
inline void NodeDef::clear_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.name_.ClearToEmpty();
  _impl_._has_bits_[0] &= ~0x00000001U;
}
inline const ::std::string& NodeDef::name() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opencv_tensorflow.NodeDef.name)
  return _internal_name();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void NodeDef::set_name(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_._has_bits_[0] |= 0x00000001U;
  _impl_.name_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:opencv_tensorflow.NodeDef.name)
}
inline ::std::string* PROTOBUF_NONNULL NodeDef::mutable_name()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::std::string* _s = _internal_mutable_name();
  // @@protoc_insertion_point(field_mutable:opencv_tensorflow.NodeDef.name)
  return _s;
}
inline const ::std::string& NodeDef::_internal_name() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.name_.Get();
}
inline void NodeDef::_internal_set_name(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_._has_bits_[0] |= 0x00000001U;
  _impl_.name_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL NodeDef::_internal_mutable_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_._has_bits_[0] |= 0x00000001U;
  return _impl_.name_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE NodeDef::release_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:opencv_tensorflow.NodeDef.name)
  if ((_impl_._has_bits_[0] & 0x00000001U) == 0) {
    return nullptr;
  }
  _impl_._has_bits_[0] &= ~0x00000001U;
  auto* released = _impl_.name_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.name_.Set("", GetArena());
  }
  return released;
}
inline void NodeDef::set_allocated_name(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    _impl_._has_bits_[0] |= 0x00000001U;
  } else {
    _impl_._has_bits_[0] &= ~0x00000001U;
  }
  _impl_.name_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.name_.IsDefault()) {
    _impl_.name_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:opencv_tensorflow.NodeDef.name)
}

// string op = 2;
inline void NodeDef::clear_op() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.op_.ClearToEmpty();
  _impl_._has_bits_[0] &= ~0x00000002U;
}
inline const ::std::string& NodeDef::op() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opencv_tensorflow.NodeDef.op)
  return _internal_op();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void NodeDef::set_op(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_._has_bits_[0] |= 0x00000002U;
  _impl_.op_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:opencv_tensorflow.NodeDef.op)
}
inline ::std::string* PROTOBUF_NONNULL NodeDef::mutable_op()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::std::string* _s = _internal_mutable_op();
  // @@protoc_insertion_point(field_mutable:opencv_tensorflow.NodeDef.op)
  return _s;
}
inline const ::std::string& NodeDef::_internal_op() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.op_.Get();
}
inline void NodeDef::_internal_set_op(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_._has_bits_[0] |= 0x00000002U;
  _impl_.op_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL NodeDef::_internal_mutable_op() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_._has_bits_[0] |= 0x00000002U;
  return _impl_.op_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE NodeDef::release_op() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:opencv_tensorflow.NodeDef.op)
  if ((_impl_._has_bits_[0] & 0x00000002U) == 0) {
    return nullptr;
  }
  _impl_._has_bits_[0] &= ~0x00000002U;
  auto* released = _impl_.op_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.op_.Set("", GetArena());
  }
  return released;
}
inline void NodeDef::set_allocated_op(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    _impl_._has_bits_[0] |= 0x00000002U;
  } else {
    _impl_._has_bits_[0] &= ~0x00000002U;
  }
  _impl_.op_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.op_.IsDefault()) {
    _impl_.op_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:opencv_tensorflow.NodeDef.op)
}

// repeated string input = 3;
inline int NodeDef::_internal_input_size() const {
  return _internal_input().size();
}
inline int NodeDef::input_size() const {
  return _internal_input_size();
}
inline void NodeDef::clear_input() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.input_.Clear();
}
inline ::std::string* PROTOBUF_NONNULL NodeDef::add_input()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::std::string* _s = _internal_mutable_input()->Add();
  // @@protoc_insertion_point(field_add_mutable:opencv_tensorflow.NodeDef.input)
  return _s;
}
inline const ::std::string& NodeDef::input(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opencv_tensorflow.NodeDef.input)
  return _internal_input().Get(index);
}
inline ::std::string* PROTOBUF_NONNULL NodeDef::mutable_input(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:opencv_tensorflow.NodeDef.input)
  return _internal_mutable_input()->Mutable(index);
}
template <typename Arg_, typename... Args_>
inline void NodeDef::set_input(int index, Arg_&& value, Args_... args) {
  ::google::protobuf::internal::AssignToString(*_internal_mutable_input()->Mutable(index), ::std::forward<Arg_>(value),
                        args... );
  // @@protoc_insertion_point(field_set:opencv_tensorflow.NodeDef.input)
}
template <typename Arg_, typename... Args_>
inline void NodeDef::add_input(Arg_&& value, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::google::protobuf::internal::AddToRepeatedPtrField(*_internal_mutable_input(),
                               ::std::forward<Arg_>(value),
                               args... );
  // @@protoc_insertion_point(field_add:opencv_tensorflow.NodeDef.input)
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>& NodeDef::input()
    const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opencv_tensorflow.NodeDef.input)
  return _internal_input();
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
NodeDef::mutable_input() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable_list:opencv_tensorflow.NodeDef.input)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_input();
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>&
NodeDef::_internal_input() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.input_;
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
NodeDef::_internal_mutable_input() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.input_;
}

// string device = 4;
inline void NodeDef::clear_device() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.device_.ClearToEmpty();
  _impl_._has_bits_[0] &= ~0x00000004U;
}
inline const ::std::string& NodeDef::device() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opencv_tensorflow.NodeDef.device)
  return _internal_device();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void NodeDef::set_device(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_._has_bits_[0] |= 0x00000004U;
  _impl_.device_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:opencv_tensorflow.NodeDef.device)
}
inline ::std::string* PROTOBUF_NONNULL NodeDef::mutable_device()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::std::string* _s = _internal_mutable_device();
  // @@protoc_insertion_point(field_mutable:opencv_tensorflow.NodeDef.device)
  return _s;
}
inline const ::std::string& NodeDef::_internal_device() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.device_.Get();
}
inline void NodeDef::_internal_set_device(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_._has_bits_[0] |= 0x00000004U;
  _impl_.device_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL NodeDef::_internal_mutable_device() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_._has_bits_[0] |= 0x00000004U;
  return _impl_.device_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE NodeDef::release_device() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:opencv_tensorflow.NodeDef.device)
  if ((_impl_._has_bits_[0] & 0x00000004U) == 0) {
    return nullptr;
  }
  _impl_._has_bits_[0] &= ~0x00000004U;
  auto* released = _impl_.device_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.device_.Set("", GetArena());
  }
  return released;
}
inline void NodeDef::set_allocated_device(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    _impl_._has_bits_[0] |= 0x00000004U;
  } else {
    _impl_._has_bits_[0] &= ~0x00000004U;
  }
  _impl_.device_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.device_.IsDefault()) {
    _impl_.device_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:opencv_tensorflow.NodeDef.device)
}

// map<string, .opencv_tensorflow.AttrValue> attr = 5;
inline int NodeDef::_internal_attr_size() const {
  return _internal_attr().size();
}
inline int NodeDef::attr_size() const {
  return _internal_attr_size();
}
inline const ::google::protobuf::Map<::std::string, ::opencv_tensorflow::AttrValue>& NodeDef::_internal_attr() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.attr_.GetMap();
}
inline const ::google::protobuf::Map<::std::string, ::opencv_tensorflow::AttrValue>& NodeDef::attr() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_map:opencv_tensorflow.NodeDef.attr)
  return _internal_attr();
}
inline ::google::protobuf::Map<::std::string, ::opencv_tensorflow::AttrValue>* PROTOBUF_NONNULL NodeDef::_internal_mutable_attr() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.attr_.MutableMap();
}
inline ::google::protobuf::Map<::std::string, ::opencv_tensorflow::AttrValue>* PROTOBUF_NONNULL NodeDef::mutable_attr()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable_map:opencv_tensorflow.NodeDef.attr)
  return _internal_mutable_attr();
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace opencv_tensorflow


// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"

#endif  // graph_2eproto_2epb_2eh
