// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: versions.proto
// Protobuf C++ Version: 6.32.1

#include "versions.pb.h"

#include <algorithm>
#include <type_traits>
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/generated_message_tctable_impl.h"
#include "google/protobuf/extension_set.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/wire_format_lite.h"
#include "google/protobuf/descriptor.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/reflection_ops.h"
#include "google/protobuf/wire_format.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"
PROTOBUF_PRAGMA_INIT_SEG
namespace _pb = ::google::protobuf;
namespace _pbi = ::google::protobuf::internal;
namespace _fl = ::google::protobuf::internal::field_layout;
namespace opencv_tensorflow {

inline constexpr VersionDef::Impl_::Impl_(
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        bad_consumers_{},
        _bad_consumers_cached_byte_size_{0},
        producer_{0},
        min_consumer_{0} {}

template <typename>
PROTOBUF_CONSTEXPR VersionDef::VersionDef(::_pbi::ConstantInitialized)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(VersionDef_class_data_.base()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(::_pbi::ConstantInitialized()) {
}
struct VersionDefDefaultTypeInternal {
  PROTOBUF_CONSTEXPR VersionDefDefaultTypeInternal() : _instance(::_pbi::ConstantInitialized{}) {}
  ~VersionDefDefaultTypeInternal() {}
  union {
    VersionDef _instance;
  };
};

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 VersionDefDefaultTypeInternal _VersionDef_default_instance_;
}  // namespace opencv_tensorflow
static constexpr const ::_pb::EnumDescriptor *PROTOBUF_NONNULL *PROTOBUF_NULLABLE
    file_level_enum_descriptors_versions_2eproto = nullptr;
static constexpr const ::_pb::ServiceDescriptor *PROTOBUF_NONNULL *PROTOBUF_NULLABLE
    file_level_service_descriptors_versions_2eproto = nullptr;
const ::uint32_t
    TableStruct_versions_2eproto::offsets[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
        protodesc_cold) = {
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::opencv_tensorflow::VersionDef, _impl_._has_bits_),
        6, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::opencv_tensorflow::VersionDef, _impl_.producer_),
        PROTOBUF_FIELD_OFFSET(::opencv_tensorflow::VersionDef, _impl_.min_consumer_),
        PROTOBUF_FIELD_OFFSET(::opencv_tensorflow::VersionDef, _impl_.bad_consumers_),
        0,
        1,
        ~0u,
};

static const ::_pbi::MigrationSchema
    schemas[] ABSL_ATTRIBUTE_SECTION_VARIABLE(protodesc_cold) = {
        {0, sizeof(::opencv_tensorflow::VersionDef)},
};
static const ::_pb::Message* PROTOBUF_NONNULL const file_default_instances[] = {
    &::opencv_tensorflow::_VersionDef_default_instance_._instance,
};
const char descriptor_table_protodef_versions_2eproto[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
    protodesc_cold) = {
    "\n\016versions.proto\022\021opencv_tensorflow\"K\n\nV"
    "ersionDef\022\020\n\010producer\030\001 \001(\005\022\024\n\014min_consu"
    "mer\030\002 \001(\005\022\025\n\rbad_consumers\030\003 \003(\005B/\n\030org."
    "tensorflow.frameworkB\016VersionsProtosP\001\370\001"
    "\001b\006proto3"
};
static ::absl::once_flag descriptor_table_versions_2eproto_once;
PROTOBUF_CONSTINIT const ::_pbi::DescriptorTable descriptor_table_versions_2eproto = {
    false,
    false,
    169,
    descriptor_table_protodef_versions_2eproto,
    "versions.proto",
    &descriptor_table_versions_2eproto_once,
    nullptr,
    0,
    1,
    schemas,
    file_default_instances,
    TableStruct_versions_2eproto::offsets,
    file_level_enum_descriptors_versions_2eproto,
    file_level_service_descriptors_versions_2eproto,
};
namespace opencv_tensorflow {
// ===================================================================

class VersionDef::_Internal {
 public:
  using HasBits =
      decltype(::std::declval<VersionDef>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(VersionDef, _impl_._has_bits_);
};

VersionDef::VersionDef(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, VersionDef_class_data_.base()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:opencv_tensorflow.VersionDef)
}
PROTOBUF_NDEBUG_INLINE VersionDef::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::opencv_tensorflow::VersionDef& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        bad_consumers_{visibility, arena, from.bad_consumers_},
        _bad_consumers_cached_byte_size_{0} {}

VersionDef::VersionDef(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const VersionDef& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, VersionDef_class_data_.base()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  VersionDef* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::memcpy(reinterpret_cast<char *>(&_impl_) +
               offsetof(Impl_, producer_),
           reinterpret_cast<const char *>(&from._impl_) +
               offsetof(Impl_, producer_),
           offsetof(Impl_, min_consumer_) -
               offsetof(Impl_, producer_) +
               sizeof(Impl_::min_consumer_));

  // @@protoc_insertion_point(copy_constructor:opencv_tensorflow.VersionDef)
}
PROTOBUF_NDEBUG_INLINE VersionDef::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        bad_consumers_{visibility, arena},
        _bad_consumers_cached_byte_size_{0} {}

inline void VersionDef::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char *>(&_impl_) +
               offsetof(Impl_, producer_),
           0,
           offsetof(Impl_, min_consumer_) -
               offsetof(Impl_, producer_) +
               sizeof(Impl_::min_consumer_));
}
VersionDef::~VersionDef() {
  // @@protoc_insertion_point(destructor:opencv_tensorflow.VersionDef)
  SharedDtor(*this);
}
inline void VersionDef::SharedDtor(MessageLite& self) {
  VersionDef& this_ = static_cast<VersionDef&>(self);
  if constexpr (::_pbi::DebugHardenVerifyHasBitConsistency()) {
    this_.VerifyHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

inline void* PROTOBUF_NONNULL VersionDef::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) VersionDef(arena);
}
constexpr auto VersionDef::InternalNewImpl_() {
  constexpr auto arena_bits = ::google::protobuf::internal::EncodePlacementArenaOffsets({
      PROTOBUF_FIELD_OFFSET(VersionDef, _impl_.bad_consumers_) +
          decltype(VersionDef::_impl_.bad_consumers_)::
              InternalGetArenaOffset(
                  ::google::protobuf::Message::internal_visibility()),
  });
  if (arena_bits.has_value()) {
    return ::google::protobuf::internal::MessageCreator::ZeroInit(
        sizeof(VersionDef), alignof(VersionDef), *arena_bits);
  } else {
    return ::google::protobuf::internal::MessageCreator(&VersionDef::PlacementNew_,
                                 sizeof(VersionDef),
                                 alignof(VersionDef));
  }
}
constexpr auto VersionDef::InternalGenerateClassData_() {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &_VersionDef_default_instance_._instance,
          &_table_.header,
          nullptr,  // OnDemandRegisterArenaDtor
          nullptr,  // IsInitialized
          &VersionDef::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<VersionDef>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &VersionDef::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<VersionDef>(), &VersionDef::ByteSizeLong,
              &VersionDef::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(VersionDef, _impl_._cached_size_),
          false,
      },
      &VersionDef::kDescriptorMethods,
      &descriptor_table_versions_2eproto,
      nullptr,  // tracker
  };
}

PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull VersionDef_class_data_ =
        VersionDef::InternalGenerateClassData_();

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
VersionDef::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&VersionDef_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(VersionDef_class_data_.tc_table);
  return VersionDef_class_data_.base();
}
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1
const ::_pbi::TcParseTable<2, 3, 0, 0, 2>
VersionDef::_table_ = {
  {
    PROTOBUF_FIELD_OFFSET(VersionDef, _impl_._has_bits_),
    0, // no _extensions_
    3, 24,  // max_field_number, fast_idx_mask
    offsetof(decltype(_table_), field_lookup_table),
    4294967288,  // skipmap
    offsetof(decltype(_table_), field_entries),
    3,  // num_field_entries
    0,  // num_aux_entries
    offsetof(decltype(_table_), field_names),  // no aux_entries
    VersionDef_class_data_.base(),
    nullptr,  // post_loop_handler
    ::_pbi::TcParser::GenericFallback,  // fallback
    #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
    ::_pbi::TcParser::GetTable<::opencv_tensorflow::VersionDef>(),  // to_prefetch
    #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
  }, {{
    {::_pbi::TcParser::MiniParse, {}},
    // int32 producer = 1;
    {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(VersionDef, _impl_.producer_), 0>(),
     {8, 0, 0, PROTOBUF_FIELD_OFFSET(VersionDef, _impl_.producer_)}},
    // int32 min_consumer = 2;
    {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(VersionDef, _impl_.min_consumer_), 1>(),
     {16, 1, 0, PROTOBUF_FIELD_OFFSET(VersionDef, _impl_.min_consumer_)}},
    // repeated int32 bad_consumers = 3;
    {::_pbi::TcParser::FastV32P1,
     {26, 63, 0, PROTOBUF_FIELD_OFFSET(VersionDef, _impl_.bad_consumers_)}},
  }}, {{
    65535, 65535
  }}, {{
    // int32 producer = 1;
    {PROTOBUF_FIELD_OFFSET(VersionDef, _impl_.producer_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
    // int32 min_consumer = 2;
    {PROTOBUF_FIELD_OFFSET(VersionDef, _impl_.min_consumer_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
    // repeated int32 bad_consumers = 3;
    {PROTOBUF_FIELD_OFFSET(VersionDef, _impl_.bad_consumers_), -1, 0, (0 | ::_fl::kFcRepeated | ::_fl::kPackedInt32)},
  }},
  // no aux_entries
  {{
  }},
};
PROTOBUF_NOINLINE void VersionDef::Clear() {
// @@protoc_insertion_point(message_clear_start:opencv_tensorflow.VersionDef)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  _impl_.bad_consumers_.Clear();
  cached_has_bits = _impl_._has_bits_[0];
  if ((cached_has_bits & 0x00000003U) != 0) {
    ::memset(&_impl_.producer_, 0, static_cast<::size_t>(
        reinterpret_cast<char*>(&_impl_.min_consumer_) -
        reinterpret_cast<char*>(&_impl_.producer_)) + sizeof(_impl_.min_consumer_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL VersionDef::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const VersionDef& this_ = static_cast<const VersionDef&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL VersionDef::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const VersionDef& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenVerifyHasBitConsistency()) {
    this_.VerifyHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:opencv_tensorflow.VersionDef)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  // int32 producer = 1;
  if ((this_._impl_._has_bits_[0] & 0x00000001U) != 0) {
    if (this_._internal_producer() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<1>(
              stream, this_._internal_producer(), target);
    }
  }

  // int32 min_consumer = 2;
  if ((this_._impl_._has_bits_[0] & 0x00000002U) != 0) {
    if (this_._internal_min_consumer() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<2>(
              stream, this_._internal_min_consumer(), target);
    }
  }

  // repeated int32 bad_consumers = 3;
  {
    int byte_size = this_._impl_._bad_consumers_cached_byte_size_.Get();
    if (byte_size > 0) {
      target = stream->WriteInt32Packed(
          3, this_._internal_bad_consumers(), byte_size, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:opencv_tensorflow.VersionDef)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t VersionDef::ByteSizeLong(const MessageLite& base) {
  const VersionDef& this_ = static_cast<const VersionDef&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t VersionDef::ByteSizeLong() const {
  const VersionDef& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:opencv_tensorflow.VersionDef)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
   {
    // repeated int32 bad_consumers = 3;
    {
      total_size +=
          ::_pbi::WireFormatLite::Int32SizeWithPackedTagSize(
              this_._internal_bad_consumers(), 1,
              this_._impl_._bad_consumers_cached_byte_size_);
    }
  }
  cached_has_bits = this_._impl_._has_bits_[0];
  if ((cached_has_bits & 0x00000003U) != 0) {
    // int32 producer = 1;
    if ((cached_has_bits & 0x00000001U) != 0) {
      if (this_._internal_producer() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_producer());
      }
    }
    // int32 min_consumer = 2;
    if ((cached_has_bits & 0x00000002U) != 0) {
      if (this_._internal_min_consumer() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_min_consumer());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void VersionDef::MergeImpl(::google::protobuf::MessageLite& to_msg, const ::google::protobuf::MessageLite& from_msg) {
  auto* const _this = static_cast<VersionDef*>(&to_msg);
  auto& from = static_cast<const VersionDef&>(from_msg);
  if constexpr (::_pbi::DebugHardenVerifyHasBitConsistency()) {
    from.VerifyHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:opencv_tensorflow.VersionDef)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  _this->_internal_mutable_bad_consumers()->MergeFrom(from._internal_bad_consumers());
  cached_has_bits = from._impl_._has_bits_[0];
  if ((cached_has_bits & 0x00000003U) != 0) {
    if ((cached_has_bits & 0x00000001U) != 0) {
      if (from._internal_producer() != 0) {
        _this->_impl_.producer_ = from._impl_.producer_;
      }
    }
    if ((cached_has_bits & 0x00000002U) != 0) {
      if (from._internal_min_consumer() != 0) {
        _this->_impl_.min_consumer_ = from._impl_.min_consumer_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(from._internal_metadata_);
}

void VersionDef::CopyFrom(const VersionDef& from) {
// @@protoc_insertion_point(class_specific_copy_from_start:opencv_tensorflow.VersionDef)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void VersionDef::InternalSwap(VersionDef* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.bad_consumers_.InternalSwap(&other->_impl_.bad_consumers_);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(VersionDef, _impl_.min_consumer_)
      + sizeof(VersionDef::_impl_.min_consumer_)
      - PROTOBUF_FIELD_OFFSET(VersionDef, _impl_.producer_)>(
          reinterpret_cast<char*>(&_impl_.producer_),
          reinterpret_cast<char*>(&other->_impl_.producer_));
}

::google::protobuf::Metadata VersionDef::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// @@protoc_insertion_point(namespace_scope)
}  // namespace opencv_tensorflow
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google
// @@protoc_insertion_point(global_scope)
PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 static ::std::false_type
    _static_init2_ [[maybe_unused]] =
        (::_pbi::AddDescriptors(&descriptor_table_versions_2eproto),
         ::std::false_type{});
#include "google/protobuf/port_undef.inc"
